/*
 ***********************************************************************************************************************
 *
 * Copyright (c) 2021-2023, Infineon Technologies AG
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 **********************************************************************************************************************/

/*******************************************************************************
**                             Author(s) Identity                             **
********************************************************************************
** Initials     Name                                                          **
** ---------------------------------------------------------------------------**
** BG           Blandine Guillot                                              **
** JO           Julia Ott                                                     **
** EE           Erich Englbrecht                                              **
*******************************************************************************/

/*******************************************************************************
**                          Revision Control History                          **
********************************************************************************
** V1.0.0: 2021-04-28, EE:   EP-694: Initial version                          **
** V1.0.1: 2021-07-27, BG:   EP-814: Added return code for TLE_init()         **
**                           Removed clearing of status registers, should be  **
**                           done in the init functions if they are passed    **
** V1.0.2: 2021-08-06, BG:   EP-695: Added an IF-condition to check if the    **
**                           data are valid, since it has been removed from   **
**                           the function ADC1_getChResult()                  **
** V1.0.3: 2021-10-06, JO:   EP-949: Corrected memory layout (IROM1 size,     **
**                           IRAM1 start and size, IRAM2 size)                **
** V1.0.4: 2021-10-15, JO:   EP-962: Updated NAC time                         **
** V1.0.5: 2021-11-11, JO:   EP-937: Updated copyright and branding           **
** V1.0.6: 2021-12-20, JO:   EP-977: Added '--diag_suppress 1609' to Asm Misc **
**                           Control to remove compiler warning               **
** V1.1.0: 2023-04-18, JO:   EP-1235: Changed to C startup file               **
** V1.2.0: 2024-06-14, JO:   EP-1480: Disabled  warning 'unsafe buffer usage' **
**                           (ARM compiler for Embedded version >6.21)        **
*******************************************************************************/

/*******************************************************************************
**                                  Abstract                                  **
********************************************************************************
** ADC1: Reading the Poti voltage via an ADC1 measurement and displaying it   **
**       on the LEDs                                                          **
********************************************************************************
** The current Poti voltage setting is read with the channel 0 of ADC1 via    **
** the input pin P2.6. The measured voltage range is divided into 10 intervals**
** 0-500mV, 500mV-1V,..., 4.5V-5V which are displayed on 9 LEDs. The ADC1     **
** sequence is triggered by software.                                         **
** Devices with 48 pins are limited to 6 LEDs and measurement is done         **
** between 0 and 3V only.                                                     **
*******************************************************************************/

/*******************************************************************************
**                                  Includes                                  **
*******************************************************************************/
#include "tle_device.h"
#include "types.h"
#include "adc1.h"
#include "gpio.h"

/*******************************************************************************
**                                 Constants                                  **
*******************************************************************************/
#define M_VOLT_500 (0x1F4)

/*******************************************************************************
**                                 Variables                                  **
*******************************************************************************/
/* Value for receiving result in mV. */
static uint16 u16_mVolt;

/*******************************************************************************
**                           Function Declarations                            **
*******************************************************************************/
void switchLedAccordingPotiVoltage(void);


sint32 main(void)
{
  sint8 s8_returnCode;
  
  /* Clear watchdog fail status */
  PMU_clrFailSafeWatchdogFailSts();

  /* Initialization of hardware modules based on Config Wizard configuration */ 
  s8_returnCode = TLE_init();
  
  if (s8_returnCode != ERR_LOG_SUCCESS)
  {
    /* Place your code here to handle an initialization error */
  }

  for (;;)
  {
    /* Main watchdog service */
    (void) PMU_serviceFailSafeWatchdog();
    
    /* Switch LEDs according to the current Poti setting (0-5V) */
    switchLedAccordingPotiVoltage();
  }
}


void switchLedAccordingPotiVoltage(void)
{
  /* Trigger ADC1 sequence 0 */
  if (ERR_LOG_SUCCESS == ADC1_startSequence(ADC1_SEQ0))
  {  
    /* Check if the data are valid */
    if (ADC1_getCh0ResultValidSts() == 1u)
    {
      /* Read digital result of Poti setting in mV */
      if (ERR_LOG_SUCCESS == ADC1_getChResult_mV(&u16_mVolt, ADC1_DCH0))
      {
        /* Calculate multiple of 500 mV */
        u16_mVolt = u16_mVolt / (uint16) M_VOLT_500;
        /* If poti voltage > 500mV then P0.1 is high, otherwise low */
        u16_mVolt > 0 ?
          GPIO_setP01State(GPIO_STATE_HIGH) : GPIO_setP01State(GPIO_STATE_LOW);
        /* If poti voltage > 1000mV then P0.2 is high, otherwise low */
        u16_mVolt > 1 ? 
          GPIO_setP02State(GPIO_STATE_HIGH) : GPIO_setP02State(GPIO_STATE_LOW);
        /* If poti voltage > 1500mV then P0.3 is high, otherwise low */
        u16_mVolt > 2 ?
          GPIO_setP03State(GPIO_STATE_HIGH) : GPIO_setP03State(GPIO_STATE_LOW);
        /* If poti voltage > 2000mV then P1.0 is high, otherwise low */
        u16_mVolt > 3 ?
          GPIO_setP10State(GPIO_STATE_HIGH) : GPIO_setP10State(GPIO_STATE_LOW);
        /* If poti voltage > 2500mV then P1.1 is high, otherwise low */
        u16_mVolt > 4 ?
          GPIO_setP11State(GPIO_STATE_HIGH) : GPIO_setP11State(GPIO_STATE_LOW);
        /* If poti voltage > 3000mV then P1.2 is high, otherwise low */
        u16_mVolt > 5 ? 
          GPIO_setP12State(GPIO_STATE_HIGH) : GPIO_setP12State(GPIO_STATE_LOW);
  #ifndef UC_FEATURE_48PIN
        /* If poti voltage > 3500mV then P0.4 is high, otherwise low */
        u16_mVolt > 6 ?
          GPIO_setP04State(GPIO_STATE_HIGH) : GPIO_setP04State(GPIO_STATE_LOW);
        /* If poti voltage > 4000mV then P0.5 is high, otherwise low */
        u16_mVolt > 7 ?
          GPIO_setP05State(GPIO_STATE_HIGH) : GPIO_setP05State(GPIO_STATE_LOW);
        /* If poti voltage > 4500mV then P0.6 is high, otherwise low */
        u16_mVolt > 8 ?
          GPIO_setP06State(GPIO_STATE_HIGH) : GPIO_setP06State(GPIO_STATE_LOW);
  #endif
      }
    }
  }
}
