/*
 ***********************************************************************************************************************
 *
 * Copyright (c) 2021, Infineon Technologies AG
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 **********************************************************************************************************************/

/*******************************************************************************
**                                  Includes                                  **
*******************************************************************************/
#include <string.h>
#include  <stdio.h>
#include "types.h"
#include "bootrom.h"
#include "aes_gcm.h"

/*******************************************************************************
**                         Private Macro Declarations                         **
*******************************************************************************/
static uint8 u8_cipherText[TLE989X_IO_BUFFER_SIZE];

/*******************************************************************************
**                        Private Function Declarations                       **
*******************************************************************************/
static uint8 getBitMask(uint8 u8_bit);
static void shiftRight(uint8 *u8p_inp);
static void xor (uint8 * X, uint8 *Y);
static void galoisFieldFMultiplier(uint8 *u8p_out, uint8 *u8p_inp1, uint8 *u8p_inp2);
static void incr(uint8 *u8p_counter);
static void GHASH(tAesGcm *sp_ctx);
static sint32 execAES(user_crypto_io_buf_t *sp_aesIO, user_crypto_fid_t fid, uint32 u32_keyId );
/*******************************************************************************
**                         Private Function Definitions                       **
*******************************************************************************/
/** \brief Performs AES encryption 
 *  \param user_crypto_io_buf_t data type pointer to input message 
 *  \param user_crypto_fid_t data type  ID of the desired operation
 *  \param  u32_keyId: Secure key ID, key ID range is 113
 *  \return sint32 0: success, <0: error codes
 */
static sint32 execAES(user_crypto_io_buf_t *sp_aesIO, user_crypto_fid_t fid, uint32 u32_keyId )
{

  sint32 s32_returnCode = 0;
  uint32 u32_encodeOutLength;
  uint32 u32_totalLength= 0;
 
  /* Initialize a ECB encryption operation */
  s32_returnCode = user_crypto_aes_start(fid, u32_keyId,  NULL_PTR);
      
  if (s32_returnCode == ERR_LOG_SUCCESS)
  {   
    /* Update the ongoing  ECB encryption  operation */
    s32_returnCode |= user_crypto_aes_update(sp_aesIO);
    u32_totalLength += *(sp_aesIO->out.length);
    u32_encodeOutLength = ENCRYPT_DATA_LENGTH;    
    
    sp_aesIO->inp.buffer = sp_aesIO->inp.buffer + AES_UPDATE_LENGTH;
    /*Reset the input length to 0 */
    sp_aesIO->inp.length = 0; 
    sp_aesIO->out.buffer = (uint8 *)(sp_aesIO->out.buffer + u32_totalLength);
    sp_aesIO->out.length = &u32_encodeOutLength;

    s32_returnCode |= user_crypto_aes_finish(sp_aesIO);      
   }
  else
  {
    /* Finish api must be called even if any other APIs returns error */
    s32_returnCode |= user_crypto_aes_finish(sp_aesIO);
  }
   
return s32_returnCode;
}

/*******************************************************************************
**                         Private Function Definitions                       **
*******************************************************************************/
/** \brief Bitmask value computation
 *  \return u8_bit: respective bitmask value
 */
static uint8 getBitMask(uint8 u8_bit)
{
  switch (u8_bit)
  {
  case 7:
    u8_bit = BIT_MASK7;
    break;
  case 6:
    u8_bit = BIT_MASK6;
    break;
  case 5:
    u8_bit = BIT_MASK5;
    break;
  case 4:
    u8_bit = BIT_MASK4;
    break;
  case 3:
    u8_bit = BIT_MASK3;
    break;
  case 2:
    u8_bit = BIT_MASK2;
    break;
  case 1:
    u8_bit = BIT_MASK1;
    break;
  case 0:
    u8_bit = BIT_MASK0;
    break;
  }
  return u8_bit;
}

/*******************************************************************************
**                         Private Function Definitions                       **
*******************************************************************************/
/** \brief GCM right shift operation  
 *  \param u8p_inp pointer to input array of data
 *  \return void
 */
static void shiftRight(uint8 *u8p_inp)
{
  uint8 u8_lastLsb = 0x00;
  uint8 u8_lsb = 0x00;
  uint8 i;

  for (i = 0; i < ENCRYPT_DATA_LENGTH; i++)
  {
    u8_lastLsb = u8_lsb;
    if (u8p_inp[i] & BIT_MASK0)
    {
      u8_lsb = BIT_MASK7;
    }
    else
    {
      u8_lsb = 0x00;
    }
    u8p_inp[i] = u8p_inp[i] >> BIT_MASK0;
    u8p_inp[i] += u8_lastLsb;
  }
}
/*******************************************************************************
**                         Private Function Definitions                       **
*******************************************************************************/
/** \brief performs xor of 2-input array  
 *  \param u8p_in pointer to input message 
 *  \param u8p_out pointer to input message, the xored output is stored 
 *  \return void
 */
static void xor (uint8 * u8p_in, uint8 *u8p_out) 
{
  uint8 i;
  for (i = 0; i < ENCRYPT_DATA_LENGTH; i++)
  {
    u8p_in[i] = u8p_in[i] ^ u8p_out[i];
  }
}

/*******************************************************************************
**                         Private Function Definitions                       **
*******************************************************************************/
/** \brief Galois Field multiplication 
* \brief Z <- 0, V <- X
* \brief for i = 0 to 127 do
* \brief  if Yi = 1 then
* \brief    Z <-  Z xor V
* \brief  end if
* \brief  if V127 = 0 then
* \brief    V <- rightshift(V )
* \brief  else
* \brief    V <- rightshift(V ) xor R
* \brief  end if
* \brief end for
* \brief return Z 
* \return u8p_outZ pointer to the output array
*/
static void galoisFieldFMultiplier(uint8 *u8p_outZ, uint8 *u8p_inpX, uint8 *u8p_inpY)
{
  uint8 u8_specialElementR[ENCRYPT_DATA_LENGTH] = {0xe1};
  uint8 intermediateValue[ENCRYPT_DATA_LENGTH] = {0};
  uint8 i;
  uint8 j;
  memset(u8p_outZ, 0, ENCRYPT_DATA_LENGTH);
  memcpy(intermediateValue, u8p_inpX, ENCRYPT_DATA_LENGTH);

  for (i = 0; i < ENCRYPT_DATA_LENGTH; i++)
  {
    for (j = 0; j < ENCRYPT_DATA_LENGTH/2; j++)
    {
      if (u8p_inpY[i] & getBitMask(7 - j))
      {
        xor(u8p_outZ, intermediateValue);
      }

      if (intermediateValue[ENCRYPT_DATA_LENGTH -1] & BIT_MASK0)
      {
        shiftRight(intermediateValue);
        intermediateValue[0] ^= u8_specialElementR[0];
      }
      else
      {
        shiftRight(intermediateValue);
      }
    }
  }
}

/*******************************************************************************
**                         Private Function Definitions                       **
*******************************************************************************/
/** \brief Counter increment operation */
static void incr(uint8 *u8_counter)
{
  /* Convert the last 4byte of input uint8 array to uint32 type and increment the value */
  uint32 temp = ((uint32)u8_counter[ENCRYPT_DATA_LENGTH -4] << 24) + ((uint32)u8_counter[ENCRYPT_DATA_LENGTH -3] << 16) + ((uint32)u8_counter[ENCRYPT_DATA_LENGTH -2] << 8) + ((uint32)u8_counter[ENCRYPT_DATA_LENGTH -1]) + 1;
  /* Update the input array with incremented value  */
  u8_counter[ENCRYPT_DATA_LENGTH -1] = (uint8)(temp);
  u8_counter[ENCRYPT_DATA_LENGTH -2] = (uint8)(temp >> 8);
  u8_counter[ENCRYPT_DATA_LENGTH -3] = (uint8)(temp >> 16);
  u8_counter[ENCRYPT_DATA_LENGTH -4] = (uint8)(temp >> 24);
}

/*******************************************************************************
**                         Private Function Definitions                       **
*******************************************************************************/
/** \brief The function GHASH is defined by GHASH(H, A, C) = Xm+n+1, where the inputs A and C are 
* \brief Xi = 0 for i = 0
* \brief    = (Xi-1 xor Ai). H for i = 1, . . . , m - 1
* \brief    = (Xm-1 xor (Am*||0^128-u )) H for i = m 
* \brief    = (Xi-1 xor Ci)  H for i = m + 1, . . . , m + n - 1
* \brief    = (Xm+n-1 xor (Cm*||0^128-u )) H for i = m + n
* \brief    = (Xm+n xor (len(A)||len(C)))  H for i = m + n + 1.
* \param sp_context, pointer to the tAesGcm strucure data type
*/
static void GHASH(tAesGcm *sp_context)
{

  uint8 u8_outputZ[ENCRYPT_DATA_LENGTH] = {0};
  uint8 u8_inpY[ENCRYPT_DATA_LENGTH] = {0};
  uint8 u8_concatenatedLength[ENCRYPT_DATA_LENGTH];
  uint32 u32_modAAD;
  uint32 u32_modCipherText;
  uint32 u32_aadLength;
  uint32 u32_cipherLength;
  uint8 u8_block[ENCRYPT_DATA_LENGTH] = {0};
  uint16 u16_cipherLength;
  uint16 u16_aadLength;
  uint32 i;
  uint32 j;

  u32_modCipherText = sp_context->s_inp.length %ENCRYPT_DATA_LENGTH;
  u32_modAAD =  sp_context->s_aad.length %ENCRYPT_DATA_LENGTH;
  /* Copy the encrypt output data for an array  */
  memcpy(u8_cipherText, sp_context->s_out.buffer, sp_context->s_out.length );
  /* Total authenticated data including the zero padding  */
  if(u32_modAAD != 0)
  {
    u32_aadLength = sp_context->s_aad.length + ENCRYPT_DATA_LENGTH - u32_modAAD ;
  }
  else
  {
    u32_aadLength = sp_context->s_aad.length;
  }
 /* If cipher text length is not multiple of 16, make it multiple of 16 by adding 0  */
  if(u32_modCipherText != 0)
  {
     u32_cipherLength = sp_context->s_out.length + ENCRYPT_DATA_LENGTH - u32_modCipherText ;
     for(i = sp_context->s_out.length; i < u32_cipherLength; i++)
     {
       u8_cipherText[i] = MEMSET_ZERO;
     }
  }
  else
  {
    u32_cipherLength = sp_context->s_out.length;
  }
  /* Perform GHASH calculation  */ 
  for(i = 0; i < u32_aadLength/ENCRYPT_DATA_LENGTH; i++)
  {
    for (j = 0; j < ENCRYPT_DATA_LENGTH; j++) 
    {
      u8_block[j] = sp_context->s_aad.buffer[(i * ENCRYPT_DATA_LENGTH) + j];
    }
    /* xor the first 16byte AAD with and  array u8_inpY*/
    xor(u8_inpY, u8_block);
    /* Perform galois field multiplication of xored output with hash key */
    galoisFieldFMultiplier(u8_outputZ, sp_context->u8_hashKey, u8_inpY);
    /* Copy the multiplied output to  u8_inpY array. Now u8_inpY array has the latest value.*/
    memcpy(u8_inpY, u8_outputZ, ENCRYPT_DATA_LENGTH);
  }

  for (i = 0; i < (u32_cipherLength / ENCRYPT_DATA_LENGTH); i++)
  {
    for (j = 0; j < ENCRYPT_DATA_LENGTH; j++)
    {
       u8_block[j] = u8_cipherText[ ((i * ENCRYPT_DATA_LENGTH) + j)];
    }
    /* Xor the first 16byte ciphertext with and  array u8_inpY*/
    xor(u8_inpY, u8_block);
    /* Perform galois field multiplication of xored output with hash key */
    galoisFieldFMultiplier(u8_outputZ, sp_context->u8_hashKey, u8_inpY);
    /* Copy the multiplied output to  u8_inpY array. Now u8_inpY array has the latest value.*/
    memcpy(u8_inpY, u8_outputZ, ENCRYPT_DATA_LENGTH);
  }

  /* Copy the encrypted text length in bits */
  u16_cipherLength = (uint16)sp_context->s_inp.length * (ENCRYPT_DATA_LENGTH/2);
  /* Copy the authenticated data length in bits */
  u16_aadLength = (uint16)sp_context->s_aad.length * (ENCRYPT_DATA_LENGTH/2);

  memset(u8_concatenatedLength, MEMSET_ZERO, ENCRYPT_DATA_LENGTH);
  /* Copy the AAD and cipher text length to an array. len(AAD) ||len(Ciphertext)*/
  for (i = 0; i < CONCATENATED_LENGTH; i++)
  {
    u8_concatenatedLength[ENCRYPT_DATA_LENGTH -1-i] = (u16_cipherLength >> 8 * i) & 0xFF;
    u8_concatenatedLength[(ENCRYPT_DATA_LENGTH/2)-1-i] = (u16_aadLength >> 8 * i) & 0xFF;
  }
  /* Xor the   */
  xor(u8_inpY, u8_concatenatedLength);
   /* Perform galois field multiplication of xored output with hash key */
  galoisFieldFMultiplier(u8_outputZ, sp_context->u8_hashKey, u8_inpY);
    /* Copy the multiplied output to  u8_inpY array. Now u8_inpY array has the latest value.*/
  memcpy(u8_inpY, u8_outputZ, ENCRYPT_DATA_LENGTH);

  /* update the u8_gHash array  */
  memcpy(sp_context->u8_gHash, u8_inpY, ENCRYPT_DATA_LENGTH);
}

/*******************************************************************************
**                         Global Function Definitions                        **
*******************************************************************************/
/** \brief GCM encryption operation
* \brief H   = E(K, 0^128)  
* \brief Y0  = IV ||0^31| 1   if len(IV ) = 96 
* \brief     = GHASH(H, {}, IV ) otherwise.
* \brief Yi  = incr(Yi-1) for i = 1, . . . , n (1)
* \brief Ci  = Pi xor E(K, Yi) for i = 1, . . . , n - 1
* \brief Ci* = Pi* xor MSBu(E(K, Yn))
* \brief  T  = MSBt(GHASH(H, A, C) xor E(K, Y0))
* \param sp_ctx, pointer to the tAesGcm strucure data type
*/
sint32 encryptOperationGCM(tAesGcm *sp_ctx)
{
  uint8 u8_modInLength;
  uint32 u32_encryptLength = ENCRYPT_DATA_LENGTH;
  sint32 s32_status = 0;
  user_crypto_io_buf_t s_aesIo;
  uint8 intermediateValue[ENCRYPT_DATA_LENGTH];
  uint8 u8_inpY0[ENCRYPT_DATA_LENGTH] = {0};
  uint8 u8_tempCipher[ENCRYPT_DATA_LENGTH] = {0};
  uint8 u8_counterBlock[ENCRYPT_DATA_LENGTH] = {0};
  uint32 u32_totalLength;
  uint8 u8_iv[GCM_DEFAULT_IV_LENGTH] = {0};
  uint32 i;
  uint32 j;

  /*  Copy the iv to local array*/
  memcpy(&u8_iv[0], sp_ctx->s_iv.buffer, sp_ctx->s_iv.length);
  memset(sp_ctx->u8_hashKey, MEMSET_ZERO, ENCRYPT_DATA_LENGTH);
  s_aesIo.inp.buffer = &sp_ctx->u8_hashKey[0];
  s_aesIo.inp.length = ENCRYPT_DATA_LENGTH;
  s_aesIo.out.buffer = &intermediateValue[0];
  s_aesIo.out.length = &u32_encryptLength;
  /* Compute the hash key , H=E(k,0^128)   */
  s32_status |= execAES(&s_aesIo, CRYPTO_ECB_ENCRYPT ,KEY_ID);

  if(s32_status ==ERR_LOG_SUCCESS)
  {
    memcpy(sp_ctx->u8_hashKey, &intermediateValue[0], ENCRYPT_DATA_LENGTH);

    /* Copy the Initialization vector to counter Y0. Y0 = IV||0^31||1  */
    memset(u8_inpY0, MEMSET_ZERO, ENCRYPT_DATA_LENGTH);
    memcpy(u8_inpY0, &u8_iv[0], GCM_DEFAULT_IV_LENGTH);
    /* Update the last byte with value 1 */
    u8_inpY0[ENCRYPT_DATA_LENGTH -1] = 1;
    
    /** Increment the counter value. Now Y1 <-Y0   */
    incr(u8_inpY0);
    
    /* Copy the new counter value to another array, which shall be encrypted using AES  */
    memcpy(u8_counterBlock, u8_inpY0, ENCRYPT_DATA_LENGTH);
    /* Check the input Data length for mod 16 */
    u8_modInLength = sp_ctx->s_inp.length % ENCRYPT_DATA_LENGTH;
    if(u8_modInLength !=0)
    {
      u32_totalLength = sp_ctx->s_inp.length + ENCRYPT_DATA_LENGTH - u8_modInLength;
    }
    else
    {
      u32_totalLength = sp_ctx->s_inp.length;
    }
    
    for (i = 0; i < u32_totalLength / ENCRYPT_DATA_LENGTH; i++)
    {   
      s_aesIo.inp.buffer = &u8_counterBlock[0];
      s_aesIo.inp.length = ENCRYPT_DATA_LENGTH;
      s_aesIo.out.buffer = &u8_tempCipher[0];
      s_aesIo.out.length = &u32_encryptLength;
      /* Encrypt the counter */
      s32_status |= execAES(&s_aesIo, CRYPTO_ECB_ENCRYPT ,KEY_ID);
      if(s32_status ==ERR_LOG_SUCCESS)
      {
        for (j = 0; j < ENCRYPT_DATA_LENGTH; j++)
        {
          /* To get the GCM encrypted output, xor the palin text with the encypted counter value*/
          sp_ctx->s_out.buffer[(i * ENCRYPT_DATA_LENGTH) + j] = sp_ctx->s_inp.buffer[(i * ENCRYPT_DATA_LENGTH) + j] ^ u8_tempCipher[j];
        } 
        /* increment the counter */       
        incr(u8_counterBlock);
      }
      else
      {
        return s32_status;
      }
    }

    /* Generate GHASH-> GHASH(A, H, C)  */
    GHASH(sp_ctx);
    
    memset(u8_inpY0, MEMSET_ZERO, ENCRYPT_DATA_LENGTH);
    /* Copy the Initialization vector to counter Y0. Y0 = IV||0^31||1  */
    memcpy(u8_inpY0, &u8_iv[0], GCM_DEFAULT_IV_LENGTH);
    /* Update the last byte with value 1 */
    u8_inpY0[ENCRYPT_DATA_LENGTH -1] = 1;
    memcpy(u8_counterBlock, u8_inpY0, ENCRYPT_DATA_LENGTH);

    s_aesIo.inp.buffer = &u8_counterBlock[0];
    s_aesIo.inp.length = ENCRYPT_DATA_LENGTH;
    s_aesIo.out.buffer = &u8_tempCipher[0];
    s_aesIo.out.length = &u32_encryptLength;
    /* Encrypt the counter */
    s32_status |= execAES(&s_aesIo, CRYPTO_ECB_ENCRYPT ,KEY_ID);
    if(s32_status ==ERR_LOG_SUCCESS)
    {
      for (j = 0; j < ENCRYPT_DATA_LENGTH; j++)
      {
          /* Generate AES GCM tag */
        sp_ctx->u8_tag[ j] = sp_ctx->u8_gHash[ j] ^ u8_tempCipher[j];
      }
    }
  }
  return s32_status;
}


