/*
 ***********************************************************************************************************************
 *
 * Copyright (c) 2020-2023, Infineon Technologies AG
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 **********************************************************************************************************************/

/*******************************************************************************
**                             Author(s) Identity                             **
********************************************************************************
** Initials     Name                                                          **
** ---------------------------------------------------------------------------**
** JO           Julia Ott                                                     **
** AA           Abhilash Abhilash                                             **
** CK           Chander Kumar                                                 **
*******************************************************************************/

/*******************************************************************************
**                          Revision Control History                          **
********************************************************************************
** V1.0.0: 2020-10-21, AA: EP-926: Initial version                            **
** V1.0.1: 2022-07-28, CK: EP-926: Modified for compilation with both ARMCC   **
**                         v5.06 and v6.16                                    **
** V1.1.0: 2023-04-18, JO: EP-1235: Changed to C startup file                 **
** V1.2.0: 2024-06-14, JO: EP-1480: Disabled warning 'unsafe buffer usage'    **
**                         (ARM compiler for Embedded version >6.21)          **
*******************************************************************************/

/*******************************************************************************
**                                  Abstract                                  **
********************************************************************************
** AES GCM: Computes the AES GCM tag and printed on UART                      **
********************************************************************************
** By starting this example the AES GCM tag is computed and printed on UART   **
** Note: GCM computation requires authenticated data (AAD) length to be       **
** multiple of 16, if not then add zero padding to AAD array and define       **
** ACTUAL_AAD_LENGTH and AAD_ZERO_PADDING_LENGTH macros accordingly.          **
*******************************************************************************/

/*******************************************************************************
**                                  Includes                                  **
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "tle_device.h"
#include "types.h"
#include "aes_gcm.h"

/*******************************************************************************
**                                 Constants                                  **
*******************************************************************************/
#define ACTUAL_AAD_LENGTH       20
#define AAD_ZERO_PADDING_LENGTH 12 

/*******************************************************************************
**                                 Variables                                  **
*******************************************************************************/
static tAesGcm s_inpMsg;
/* Input Data */
static uint8 u8_plainText[60] = {0xd9, 0x31, 0x32, 0x25, 0xf8, 0x84, 0x06, 0xe5, 0xa5, 0x59, 0x09, 0xc5, 0xaf, 0xf5, 0x26, 0x9a, 0x86, 0xa7, 0xa9, 0x53, 0x15, 0x34, 0xf7, 0xda, 0x2e, 0x4c, 0x30, 0x3d, 0x8a, 0x31, 0x8a, 0x72, 0x1c, 0x3c, 0x0c, 0x95, 0x95, 0x68, 0x09, 0x53, 0x2f, 0xcf, 0x0e, 0x24, 0x49, 0xa6, 0xb5, 0x25, 0xb1, 0x6a, 0xed, 0xf5, 0xaa, 0x0d, 0xe6, 0x57, 0xba, 0x63, 0x7b, 0x39};
/* Initialization vector, Note: Only 12Bytes of IV supported in the current implementation */ 
static  uint8 u8_initVector[GCM_DEFAULT_IV_LENGTH] = {0xca, 0xfe, 0xba, 0xbe, 0xfa, 0xce, 0xdb, 0xad, 0xde, 0xca, 0xf8, 0x88}; 
/* Authenticated data for ghash computation */
static  uint8 autheticatedData[ACTUAL_AAD_LENGTH + AAD_ZERO_PADDING_LENGTH] =  {0xfe, 0xed, 0xfa, 0xce, 0xde, 0xad, 0xbe, 0xef, 0xfe, 0xed, 0xfa, 0xce, 0xde, 0xad, 0xbe, 0xef, 0xab, 0xad, 0xda, 0xd2, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0};
/* Array to store encrypted data*/
static uint8 u8_encryptOutput[TLE989X_IO_BUFFER_SIZE];  


sint32 main(void)
{
  sint8 s8_returnCode;
  uint8 i;
  /* Clear watchdog fail status */
  PMU_clrFailSafeWatchdogFailSts();
  /* Initialization of hardware modules based on Config Wizard configuration */
  s8_returnCode = TLE_init();
  if (s8_returnCode != ERR_LOG_SUCCESS)
  {
  /* Place your code here to handle an initialization error */
  }

  s_inpMsg.s_inp.buffer = (uint8*)&u8_plainText[0];
  s_inpMsg.s_inp.length = sizeof(u8_plainText);
  s_inpMsg.s_iv.buffer = (uint8*) &u8_initVector;
  s_inpMsg.s_out.buffer = (uint8*)&u8_encryptOutput;
  s_inpMsg.s_out.length =  sizeof(u8_plainText);
  s_inpMsg.s_aad.buffer =(uint8*)&autheticatedData;
  s_inpMsg.s_iv.length = sizeof(u8_initVector);
  s_inpMsg.s_aad.length = ACTUAL_AAD_LENGTH;  
      
  /**@brief  Calls AES GCM encryption is used in this example  */ 
  if (encryptOperationGCM(&s_inpMsg) == ERR_LOG_SUCCESS)
  { 
    /* Send out response  via stdout */
    printf("\n --> Tag 0x%x", s_inpMsg.u8_tag[0]);
    for(i = 1; i< 16; i ++)
    {
      printf(" 0x%x", s_inpMsg.u8_tag[i]);
    }
    printf("\n");
  }
  else
  {
    printf("\n GCM Computation Failed");
  }
  for (;;)
  {
    /* Main watchdog service */
    (void) PMU_serviceFailSafeWatchdog();

  }
}
