/*
 ***********************************************************************************************************************
 *
 * Copyright (c) 2022-2023, Infineon Technologies AG
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 **********************************************************************************************************************/

/*******************************************************************************
**                             Author(s) Identity                             **
********************************************************************************
** Initials     Name                                                          **
** ---------------------------------------------------------------------------**
** JO           Julia Ott                                                     **
** CK           Chander Kumar                                                 **
*******************************************************************************/

/*******************************************************************************
**                          Revision Control History                          **
********************************************************************************
** V1.0.0: 2022-08-25, JO, CK: EP-1243: Initial version of bootloader         **
** V1.1.0: 2023-04-18, JO:     EP-1235: Changed to C startup file, adapted    **
**                             range check for stack pointer                  **
** V1.2.0: 2024-06-14, JO:     EP-1480: Disabled warning 'unsafe buffer usage'**
**                             (ARM compiler for Embedded version >6.21)      **
*******************************************************************************/

/*******************************************************************************
**                                  Abstract                                  **
********************************************************************************
** Bootloader example                                                         **
********************************************************************************
** Flash both Bootloader and Application example to the chip.                 **
** Bootloader toggels P0.1.                                                   **
** On MON1 press, the Bootloader jumps to the application (P0.1 not toggled   **
** anymore).                                                                  **
** Application toggels P0.2.                                                  **
*******************************************************************************/

/*******************************************************************************
**                                  Includes                                  **
*******************************************************************************/
#include "tle_device.h"
#include "types.h"
#include "gpio.h"

/*******************************************************************************
**                                 Constants                                  **
*******************************************************************************/
#define USER_APPLICATION_VTAB_ADDRESS (UC_FLASH1_START + 0x100U)  /* for Application example */
#define RAM_START_ADRESS              (UC_PSRAM_START)
#define RAM_END_ADDRESS               (UC_DSRAM_START + UC_DSRAM_SIZE)
#define FLASH1_START_ADDRESS          (UC_FLASH1_START)
#define FLASH1_END_ADDRESS            (UC_FLASH1_START + UC_FLASH1_SIZE)

/*******************************************************************************
**                                 Variables                                  **
*******************************************************************************/
static bool b_jumpToApplication = false;

/*******************************************************************************
**                           Function Declarations                            **
*******************************************************************************/
void MON1_RI_Int(void);
__attribute__( ( naked, noreturn ) ) void BootJumpASM( uint32_t SP, uint32_t RH );


sint32 main(void)
{
  sint8 s8_returnCode;
  uint32 u32_tick;
  uint32* Address;
  
  u32_tick = 0;
  Address = (uint32*)USER_APPLICATION_VTAB_ADDRESS;
  
  /* Clear watchdog fail status */
  PMU_clrFailSafeWatchdogFailSts();

  /* Initialization of hardware modules based on Config Wizard configuration */
  s8_returnCode = TLE_init();
  
  if (s8_returnCode != ERR_LOG_SUCCESS)
  {
    /* Place your code here to handle an initialization error */
  }

  for (;;)
  {
    /* Main watchdog service */
    (void) PMU_serviceFailSafeWatchdog();
    GPIO->P0_OMR.reg = 0x00040004;

    u32_tick++;
    if (u32_tick > 100000)
    {
      GPIO_setP01State(GPIO_STATE_TOGGLE);
      u32_tick = 0;
    }
    
    if(b_jumpToApplication == true)
    {
      /* Check if stack pointer is in one of the 2 RAMs and reset handler is within Flash1 range */ 
      if(Address[0] >= RAM_START_ADRESS && 
				 Address[0] <= (RAM_END_ADDRESS + 0x1U) && 
			   Address[1] >= FLASH1_START_ADDRESS && 
			   Address[1] < FLASH1_END_ADDRESS)
      {
        /* Service Watchdog */
        PMU_serviceFailSafeWatchdogSOW();
        GPIO->P1_OMR.reg = 0x00010001;
        
        /* Disable all interrupts */
        __disable_irq();
        
        /* point VTOR to new vector table */
        CPU->VTOR.reg = USER_APPLICATION_VTAB_ADDRESS;
        
        /*Jump to new application */
        BootJumpASM( Address[ 0 ], Address[ 1 ] ) ;
      }
      else
      {
        /* Error hadling in case of wrong stack pointer or reset handler address */
      }
    }
  }
}


/* Assembly function to jump execution to a location */
__attribute__( ( naked, noreturn ) ) void BootJumpASM( uint32_t SP, uint32_t RH )
{
  __asm("MSR      MSP,r0");
  __asm("BX       r1");
}


/* user-defined call back function for rising edge on MON1 pin */
void MON1_RI_Int(void)
{
  b_jumpToApplication = true;
}
