/**************************************************************************************************
*                _  _               ___             _                                             *
*               (_)| |__   _ __    / _ \ _ __ ___  | |__    /\  /\                                *
*               | || '_ \ | '__|  / /_\/| '_ ` _ \ | '_ \  / /_/ /                                *
*               | || | | || |    / /_\\ | | | | | || |_) |/ __  /                                 *
*               |_||_| |_||_|    \____/ |_| |_| |_||_.__/ \/ /_/                                  *
*                                                                                                 *
*    ihr GmbH                                                                                     *
*    Airport Boulevard B210                                                                       *
*    77836 Rheinmuenster - Germany                                                                *
*    http://www.ihr.de                                                                            *
*    Phone +49(0) 7229-18475-0                                                                    *
*    Fax   +49(0) 7229-18475-11                                                                   *
***************************************************************************************************
*                                                                                                 *
*       Revision:          $Rev:: 301         $                                                   *
*       Last Modtime:      $Date:: 2022-06-23#$                                                   *
*       Last Modification: $Author:: PhilippKoch                                                $ *
*                                                                                                 *
**************************************************************************************************/
/**
@file         main.c
@brief        Will be called after startup. Initializes HW and has main loop of program
*/

/* ========== [includes] ======================================================================= */

#include "main.h"
#include "tle_device.h"
#include "Can_Api.h"
#include "Com_Api.h"
#include "com_cfg.h"

/**************************************************************************************************
*                                                                                                 *
* *** Quick start manual for IHR CAN driver evaluation ***                                        *
*                                                                                                 *
* The IDE project and the CAN driver are pre-configured to immediately work on the                *
* evaluation hardware:                                                                            *
*    - TLE988x_TLE989x Evalboard Base board Rev 1.0                                               *
*    - TLE988x_TLE989x_TQFP48_Socketboard_rev1_0                                                  *
*    - Chip TLE9893_2QTW62S                                                                       *
*    - UART/CAN clock 80 MHz (preconfigured in ConfigWizard)                                      *
*    - SysTick shall be triggered in 1 ms cycle (preconfigured in ConfigWizard)                   *
*                                                                                                 *
*    - CAUTION: The socket board might be delivered with a different oscillator! If the           *
*      driver does not communicate with the expected baud rate or the system initialization       *
*      is getting stuck in the PLL init section, please check the oscillator settings.            *
*                                                                                                 *
* The CAN driver will immediately start communicating with the following settings                 *
*    - Nominal baudrate 500 kBit/s, SP = 85%  (derived from 80 MHz CAN clock)                     *
*      BrDiv = 8, Tseg1Q = 16, Tseg2Q = 3, SjwQ = 3                                               *
*    - Data baudrate 2000 kBit/s, SP = 85%  (derived from 80 MHz CAN clock)                       *
*      DBrDiv = 2, DTseg1Q = 16, DTseg2Q = 3, DSjwQ = 3                                           *
*                                                                                                 *
* The CAN driver can cyclically send the following two frames                                     *
*    - ID 0x234:    ClassicCAN,  8 Byte, StandardID (cyclic transmission is enabled per default)  *
*                                                                                                 *
*      Optionally, the following frame can be activated (see com_cfg.c line 71...73)              *
*    - ID 0x234567: CAN-FD,     64 Byte, ExtendedID, Baudrate switching active                    *
*                                                                                                 *
* The CAN driver will listen for the following two frames                                         *
*    - ID 0x123:    ClassicCAN,  8 Byte, StandardID                                               *
*    - ID 0x123456: CAN-FD,     64 Byte, ExtendedID, Baudrate switching active                    *
*                                                                                                 *
* The demo application code in main() shows how to use the basic CAN driver interfaces            *
*    - Frame based data access using Com_ReceiveData()/Com_SendData()                             *
*    - Signal based data access using Com_ReceiveSignal()/Com_SendSignal()                        *
*                                                                                                 *
* The features of the CAN driver demo are restricted                                              *
*    - reception and transmission is limited to two frames Rx and Tx each                         *
*    - FIFO operation is not supported                                                            *
*    - interrupt operation is not supported                                                       *
*    - the CAN driver's callout functions are not supported                                       *
*                                                                                                 *
* The communication settings of the demo driver can be adapted. Please find more detailed         *
* information in                                                                                  *
*    - The CAN driver's interfaces (not all of them are available in the demo):                   *
*      Can_Api.h, Com_Api.h                                                                       *
*    - The CAN driver's configuration (restrictions are commented there)                          *
*      can_cfg.c/h, com_cfg.c/h                                                                   *
*                                                                                                 *
* The CAN Driver demo is intended to allow evaluation of the CAN driver and its integration.      *
* For usage in a productive application a license must be acquired. Please get in touch with      *
* us at sales@ihr.de                                                                              *
*                                                                                                 *
**************************************************************************************************/

/* ========== [defines] ======================================================================== */

#define COM_RX_TIMESLICE_TASK_MS    1u

/* ========== [macros] ========================================================================= */

/* empty */

/* ========== [unions, structs and enums] ====================================================== */

/* empty */

/* ========== [internal function declarations] ================================================= */

/* empty */

/* ========== [variable declarations] ========================================================== */

volatile uint8 tmrFlag;

static uint8  DemoBuffer[8] = { 1u, 2u, 3u, 4u, 5u, 6u, 7u, 8u};
static uint8  DemoVariable = 0u; 
static uint32 DemoCounter = 0u;

/* ========== [function bodies] ================================================================ */

int main(void)
{
   uint16 timeslices = 0u;
   sint8 s8_returnCode;
   s8_returnCode = ERR_LOG_SUCCESS;

   /* clear Watchdog fail status */
   PMU_clrFailSafeWatchdogFailSts();

  /* Initialization of hardware modules based on Config Wizard configuration */
  s8_returnCode = TLE_init();
  
  if (s8_returnCode != ERR_LOG_SUCCESS)
  {
    /* Place your code here to handle an initialization error */
  }

   /* 48pin only, FIFO pull-up enabled */
   PMU->MISC_CTRL.bit.FI_PU_EN = 1;

   /* clear fail safe status flags */
   while (PMU->FS_STS.reg != 0)
   {
      PMU->FS_STS_CLR.reg = 0xFFFFFFFF;
   }
   while (PMU->FS_SSD.reg != 0)
   {
      PMU->FS_SSD_CLR.reg = 0xFFFFFFFF;
   }
   /* clear bridge driver status flags */
   BDRV->STSCLR.reg = 0xFFFFFFFF;

   /* Information: Scheduling will use the existing SysTick 1 ms cycle */
   
   /* Configure GPIO for debug pins */
   GPIO->P0_DIR.bit.DIR2 = 1u;
   GPIO->P0_DIR.bit.DIR3 = 1u;
   GPIO->P1_DIR.bit.DIR0 = 1u;
   
   tmrFlag = 0u;
   
   Can_Init();
   Com_Init();
   
   /* Request Full communication */
   Com_EnableReceptionDM(0);
   Com_EnableTransmission(STD_ON);
   
   /* Debug LED: End of init. */
   GPIO->P0_OUT.bit.PO2 = 1u;
      
   for (;;)
   {
      (void)PMU_serviceFailSafeWatchdog();
		 
      if (0u != tmrFlag)
      {
         tmrFlag = 0u;
         
         /* Debug LED: Time slice */
         GPIO->P0_OUT.bit.PO3 ^= 1u;
         
         Com_MainFunctionTx();
         Com_MainFunctionRx(COM_RX_TIMESLICE_TASK_MS);

         /* Example for frame based access:
            Read data (first four bytes) from Rx message and return them 
            in the Tx message. */
         (void)Com_ReceiveData(CanRx_DemoRxClassicMsg, DemoBuffer, 4u);
         (void)Com_SendData   (CanTx_DemoTxClassicMsg, DemoBuffer, 4u);

         /* Example for signal based access:
            Read data from Rx signal CanRxSig_11_Fd_u8_1 and return it 
            in Tx signal CanTxSig_11_Fd_u8_1. */
         (void)Com_ReceiveSignal(CanRxSig_11_Fd_u8_1, &DemoVariable);
               Com_SendSignal   (CanTxSig_11_Fd_u8_1, &DemoVariable);
         
         /* Slow Debug LED (1 sec toggle) */
         timeslices++;
         if(timeslices > 1000u)
         {
            timeslices = 0u;
            GPIO->P1_OUT.bit.PO0 ^= 1u;
            
            /* Increment demo counter */
            DemoCounter++;
            
            if(DemoCounter > 1000000u)
            {
               DemoCounter = 0u;
            }
            
            /* Write counter to u32 demo signals in both TX messages. */
            Com_SendSignal(CanTxSig_3_Cl_u32_1,   &DemoCounter);
            Com_SendSignal(CanTxSig_29_Fd_u32_01, &DemoCounter);
         }
      }
   }
}

/* Callback from RTE */
void SysTickIsr(void)
{
   tmrFlag = 1u;
}

/* ========== [end of file] ==================================================================== */
