/*
 ***********************************************************************************************************************
 *
 * Copyright (c) 2021-2023, Infineon Technologies AG
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 **********************************************************************************************************************/

/*******************************************************************************
**                             Author(s) Identity                             **
********************************************************************************
** Initials     Name                                                          **
** ---------------------------------------------------------------------------**
** MK           Martin Kilian                                                 **
** JO           Julia Ott                                                     **
*******************************************************************************/

/*******************************************************************************
**                          Revision Control History                          **
********************************************************************************
** V0.1.0: 2021-05-14, MK, JO: Initial version of the example (C code and     **
**                             Config Wizard GUI for TLE987x)                 **
** V1.0.0: 2022-02-02, JO:     Ported example from TLE987x to TLE989x         **
** V1.1.0: 2023-04-18, JO:     EP-1235: Changed to C startup file             **
** V1.2.0: 2024-06-14, JO:     EP-1480: Disabled warning 'unsafe buffer usage'**
**                             (ARM compiler for Embedded version >6.21)      **
*******************************************************************************/

/*******************************************************************************
**                                  Abstract                                  **
********************************************************************************
**                            MCTRL Example: Hall                             **
********************************************************************************
** This example turns a motor with Hall sensors.                              **
** There are two options:                                                     **
** 1. Hall pattern known: In Config Wizard, select the option "Hall pattern   **
**    known" and enter the Hall Pattern. Connect the phases and Hall signals  **
**    accordingly.                                                            **
** 2. Hall pattern not known: The example offers the option to detect the     **
**    Hall pattern. In Config Wizard, select the option "Hall pattern not     **
**    known - to be measured" to enable the Hall pattern measurement.         **
**                                                                            **
** By default, the option "Hall Pattern not known - to be measured" is        **
** selected.                                                                  **
**                                                                            **
** Note: In case the Motor doesn't run smoothly, it might be that the Hall    **
** Pattern were not detected correctly. In this case, please reset the        **
** device and try again.                                                      **
**                                                                            **
** The potentiometer sets the speed of the motor, both directions are         **
** supported.                                                                 **
**                                                                            **
** The example is set up for the motor Nanotec DB42S03-KALP.                  **
** Other motors might need an update of the Startup Config in Config Wizard.  **
*******************************************************************************/

/*******************************************************************************
**                                  Includes                                  **
*******************************************************************************/
#include "tle_device.h"
#include "bldc_motor_lib.h"
#include "stdlib.h"

/*******************************************************************************
**                                 Constants                                  **
*******************************************************************************/
#define FACTOR_POTI 3  /* 5000mV*FACTOR_POTI = rpm */
#define FACTOR_VOLT 10 /* Max voltage for Motor dutycycle */

/*******************************************************************************
**                                 Variables                                  **
*******************************************************************************/
static uint16 poti_value_mV    = 0u;

/*******************************************************************************
**                           Function Declarations                            **
*******************************************************************************/
void systick_1ms(void);
void Main_Motor_Handling(void);
void printHallPtn(void);


int main(void) 
{
  sint8 s8_returnCode;

  /* Clear watchdog fail status */
  PMU_clrFailSafeWatchdogFailSts();

  /* Initialization of hardware modules based on Config Wizard configuration */
  s8_returnCode = TLE_init();
  
  if (s8_returnCode != ERR_LOG_SUCCESS)
  {
    /* Place your code here to handle an initialization error */
  }
  
  Motor_State_Set(MOTOR_STATE_STOP); /* Initialize motor state */
  
  ADC1_startSequence(ADC1_SEQ0);

  for (;;)
  {
    /* Main watchdog service */
    (void) PMU_serviceFailSafeWatchdog();
  }
}


/* Systick Interrupt for Speedcontrol */
void systick_1ms(void) 
{
  /* Main_Motor_Handling runs in the 1ms Systick Interrupt */
  Main_Motor_Handling(); 
}


/* Speedcontrol Routine 1ms*/
void Main_Motor_Handling(void)
{
  sint16 s16_poti_value_scaled;
  uint32 u32_motor_dir;
  
  ADC1_getChResult_mV(&poti_value_mV, ADC1_DCH0); /* get poti value 0..5000mV */
  ADC1_startSequence(ADC1_SEQ0);
	
  /* Limiter --> 0...5000 */
  if(poti_value_mV > ADC1_VREF_5000mV) 
  { 
    poti_value_mV = ADC1_VREF_5000mV;
  }
	
  /* Scale --> -2500...2500 */
  s16_poti_value_scaled = (sint16)poti_value_mV - (sint16)(ADC1_VREF_5000mV/2);
  
  /* Define direction (-2500...0: DIR_BACKWARD, 0...2500: DIR_FORWARD) */
  if (s16_poti_value_scaled < 0)
  {
    u32_motor_dir = DIR_BACKWARD;
  }
  else
  {
    u32_motor_dir = DIR_FORWARD;
  }
	
  if((s16_poti_value_scaled < -200) || (s16_poti_value_scaled > 200)) /* start motor at potentiometer values > 400mV */
  {
    switch(Motor_State_Get())
    {
      case MOTOR_STATE_STOP:     Motor_Init(u32_motor_dir);
																 Motor_Startup();
																 break;
      case MOTOR_STATE_RUNNING : 
                                 Motor_Direction_Set(u32_motor_dir);
                                 Motor_Dutycycle_Set((uint32)abs(s16_poti_value_scaled)/3); /* Set Dutycycle 0% to 100.0% */
                                 break;
      default                  : break;
		}
  }
  else /* stop motor at potentiometer values <= 400mV */
	{
      Motor_Stop();
	}
}
