/*
 ***********************************************************************************************************************
 *
 * Copyright (c) 2015, Infineon Technologies AG
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 **********************************************************************************************************************/

/********************* Pattern *******************************
 *         ________________________                          *
 * Hall 1                          |_______________________  *
 *                          _______________________          *
 * Hall 2  ________________|                       |_______  *
 *         ________                         _______________  *
 * Hall 3          |_______________________|                 *
 *                                                           *
 *         : LS1   : Z     : PWM   : PWM   : Z     : LS1   : *
 * Phase1  :       :-------:|||||||:|||||||:-------:       : *
 *         :_______:       :       :       :       :_______: *
 *         :       :       :       :       :       :       : *
 *         : PWM   : PWM   : Z     : LS2   : LS2   : Z     : *
 * Phase2  :|||||||:|||||||:-------:       :       :-------: *
 *         :       :       :       :_______:_______:       : *
 *         :       :       :       :       :       :       : *
 *         : Z     : LS3   : LS3   : Z     : PWM   : PWM   : *
 * Phase3  :-------:       :       :-------:|||||||:|||||||: *
 *         :       :_______:_______:       :       :       : *
 *         : Ptn_1 : Ptn_2 : Ptn_3 : Ptn_4 : Ptn_5 : Ptn_6   *
 *                                                           *
 * Switch  |       |       |       |       |       |       | *
 * event   ^       ^       ^       ^       ^       ^       ^ *
 *                                                           *
 *************************************************************/
 
/*******************************************************************************
**                                  Includes                                  **
*******************************************************************************/
#include "bldc_motor_lib.h"
#include "tle_device.h"
#include "timer_lib.h"

/*******************************************************************************
**                        Static variable Definitions                         **
*******************************************************************************/
/* struct for startup variables */
volatile struct Startup_Config Startup_Config; 

/* struct for motor variables */
volatile struct Motor Motor;

/* struct for Hall variables */
volatile struct Hall Hall;

/* Array with driver patterns */
static uint32 MCMOUT_Ptn[6] = { MCTRL_LIB_MCMOUT_PTN1, MCTRL_LIB_MCMOUT_PTN2, MCTRL_LIB_MCMOUT_PTN3, MCTRL_LIB_MCMOUT_PTN4, MCTRL_LIB_MCMOUT_PTN5, MCTRL_LIB_MCMOUT_PTN6 }; 


/*******************************************************************************
**                         Private Type Declarations                          **
*******************************************************************************/
typedef void (*StaticEn_Pattern_Functions)(void); 

/*******************************************************************************
**                       Private Function Declarations                        **
*******************************************************************************/
uint32 Update_Dutycycle(uint16 dutycycle, uint32 pattern, uint32 direction);
void Ptn1_StaticEn(void);
void Ptn2_StaticEn(void);
void Ptn3_StaticEn(void);
void Ptn4_StaticEn(void);
void Ptn5_StaticEn(void);
void Ptn6_StaticEn(void);
void Hall_Handling(void);

/*******************************************************************************
**                          ISR Function Definitions                          **
*******************************************************************************/
void COMMUTATION_TIMER_ISR(void);
void Startup_Handling(void);
void Commutation_Handling(void);
void Wrong_Hall_Event(void);
void Correct_Hall_Event(void);


/*******************************************************************************
**                        Private Function Definitions                        **
*******************************************************************************/
/* Functions enabling LSx by setting the Dutycycle to min value, use define from Config Wizard */
void Ptn1_StaticEn(void){ MCTRL_LIB_STATIC_EN_PTN1; } /* minimal compare value -> 0% PWM DC  */
void Ptn2_StaticEn(void){ MCTRL_LIB_STATIC_EN_PTN2; } /* minimal compare value -> 0% PWM DC  */
void Ptn3_StaticEn(void){ MCTRL_LIB_STATIC_EN_PTN3; } /* minimal compare value -> 0% PWM DC  */
void Ptn4_StaticEn(void){ MCTRL_LIB_STATIC_EN_PTN4; } /* minimal compare value -> 0% PWM DC  */
void Ptn5_StaticEn(void){ MCTRL_LIB_STATIC_EN_PTN5; } /* minimal compare value -> 0% PWM DC  */
void Ptn6_StaticEn(void){ MCTRL_LIB_STATIC_EN_PTN6; } /* minimal compare value -> 0% PWM DC  */

/* Function pointer array to switch static phases */
static StaticEn_Pattern_Functions StaticEn_Pattern[6] = {&Ptn1_StaticEn,&Ptn2_StaticEn,&Ptn3_StaticEn,&Ptn4_StaticEn,&Ptn5_StaticEn,&Ptn6_StaticEn}; // sequence matches the  CCU pattern

/*******************************************************************************
**                        Private Function Definitions                        **
*******************************************************************************/
/* Update Dutycycle and PWM Pattern for the MOSFET bridge */
uint32 Update_Dutycycle(uint16 dutycycle, uint32 pattern, uint32 direction) //Update_Dutycycle accoring to the active pwm phase
{
  uint16 curh = 0u;     /* variable for current Hall pattern  */
  uint16 exph = 0u;     /* variable for expected Hall pattern */
  uint16 mcmout = 0u;   /* variable for multichannel pattern  */
  
  curh =   Hall.patterns[(( pattern             + (direction>>2)) % ACTIVE_PATTERN_MAX )]; /* current hall pattern  */
  exph =   Hall.patterns[(( pattern + direction + (direction>>2)) % ACTIVE_PATTERN_MAX )]; /* expected hall pattern */
  
  mcmout = (uint16)MCMOUT_Ptn[   (( pattern + direction ) % ACTIVE_PATTERN_MAX )]; /* Driver pattern */
  
  CCU7_setMultiChModePWMPatternShadow((uint8)mcmout); /* write Hall and PWM patterns in shadow register */
  CCU7_setHallPatternCurrentShadow((uint8)curh);
  CCU7_setHallPatternExpectedShadow((uint8)exph);
  
  CCU7_setCC70AValShadow(dutycycle); /* Set Dutycycle for Phase 1 */
  CCU7_setCC71AValShadow(dutycycle); /* Set Dutycycle for Phase 2 */
  CCU7_setCC72AValShadow(dutycycle); /* Set Dutycycle for Phase 3 */
  
  StaticEn_Pattern[(( pattern + direction) % ACTIVE_PATTERN_MAX)](); /* LSx will be switched according to enabled pattern */
  
  CCU7->TCTR4.bit.T12STR = 1u;  /* Shadow transfer request for dutycycle update */
  CCU7->MCMOUTS.bit.STRHP = 1u; /* Shadow transfer request for Hall pattern */    
  /* Shadow transfer for MCM takes place at T12 ZM as configured in CCU7->MCMCTR.bit.SWSYN */
  return (( pattern + direction  ) % ACTIVE_PATTERN_MAX ); /* Increment and limit motor_active_pattern */
}

/* ISR: Commutation timer callback */
void COMMUTATION_TIMER_ISR(void)
{
  switch(Motor.state)
  {
    case MOTOR_STATE_STARTUP: Startup_Handling();     /* Open Loop Startup */
                              break;
    case MOTOR_STATE_RUNNING: Commutation_Handling(); /* Standard commutation */
                              break;
    default:                  break;
  }  
}

/* Commutation timer callback --> startup handling */
void Startup_Handling(void)
{
  /* align motor */
  if (Motor.dutycycle < MCTRL_LIB_STARTUP_DUTYCYCLE)
  {
    Motor.dutycycle += 10;
    COMMUTATION_Timer_Handling(Startup_Config.speed_count);
  }
  else
  {
     Motor_Step();  
    
    #if (BLDC_HALL_PTN_KNOWN == 0)
      if(Startup_Config.speed_count > Startup_Config.speed_end) /* Startup phase */
      {
        Startup_Config.speed_count -= Startup_Config.ramp;  /* decrease time between steps */
        Motor.value_60_degree = Startup_Config.speed_count;
    
        COMMUTATION_Timer_Handling(Startup_Config.speed_count);
      }
      else
      {
        CCU7_setMultiChModePWMPatternShadow(MCTRL_LIB_MCMOUT_Z); /* Z Mode (Motor is running free) */
        Motor.dutycycle = MCTRL_LIB_STARTUP_DUTYCYCLE;
        COMMUTATION_Timer_Handling(Startup_Config.speed_count);
        
        /*Hall Interrupt Clear & Enable */
        CCU7_clrHallEventWrongIntSts();
        CCU7_enHallEventWrongInt();
        CCU7_clrHallEventCorrectIntSts();
        CCU7_enHallEventCorrectInt();
      }
    #else
      /* When the Hall pattern are known, perform also start with Hall pattern */
      if(Startup_Config.speed_count > Startup_Config.speed_end) /* Startup phase */
      {
        Startup_Config.speed_count -= Startup_Config.ramp;  /* decrease time between steps */
        Motor.value_60_degree = Startup_Config.speed_count;
        
        COMMUTATION_Timer_Handling(Startup_Config.speed_count);
    
        /*Hall Interrupt Clear & Enable */
        CCU7_clrHallEventWrongIntSts();
        CCU7_enHallEventWrongInt();
        CCU7_clrHallEventCorrectIntSts();
        CCU7_enHallEventCorrectInt();
      }
      else
      {
        CCU7_setMultiChModePWMPatternShadow(MCTRL_LIB_MCMOUT_Z); /* Z Mode (Motor is running free) */
        Motor.dutycycle = MCTRL_LIB_STARTUP_DUTYCYCLE;
        COMMUTATION_Timer_Handling(Startup_Config.speed_count);
        
        /*Hall Interrupt Clear & Enable */
        CCU7_clrHallEventWrongIntSts();
        CCU7_enHallEventWrongInt();
        CCU7_clrHallEventCorrectIntSts();
        CCU7_enHallEventCorrectInt();
      }    
    #endif
  }
}

/* Commutation timer callback --> commutation handling */
void Commutation_Handling(void)
{
  Motor_Step();
}

/*********************************************************************************************************/
/**                                           HALL                                                      **/
/*********************************************************************************************************/

/* ISR: Hall Sensor Handling */
void Wrong_Hall_Event(void) 
{
  DEBUG_PORT_TOGGLE3();
  #if (BLDC_HALL_PTN_KNOWN == 0)
    /* Update Hall Pattern in case they were not given in Config Wizard */
    Hall.patterns[(Motor.active_pattern + (Motor.dir>>2)) %ACTIVE_PATTERN_MAX] = (CCU7->CMPSTAT.reg >> 3u) & 7u; /* Read Hall signals directly */
    Hall_Handling();
  #endif
}

/* ISR: Hall Pattern is as expected */
void Correct_Hall_Event(void)
{
  DEBUG_PORT_TOGGLE2();
  Hall_Handling();
} 

/* Get Timer values, update variables and switch pattern */
void Hall_Handling(void)
{
  Hall.value_60_degree = HALL_Timer_Handling(0); /* Measure Time between Hall Pulses  */
  
  if(Motor.state == MOTOR_STATE_STARTUP) /* if the Motor is in Startup phase  */
  {
    Motor.state = MOTOR_STATE_RUNNING; /* Motor state -> running if hall pulses recieved  */
  }
    
  Motor.value_60_degree = Hall.value_60_degree; /* in Hall mode, Motor speed is equal hall speed  */
    
  COMMUTATION_Timer_Handling((Hall.value_60_degree*Hall.delay)/60 + 1);  /* Next commutation after Hall Delay  */
}

/*********************************************************************************************************/
/**                                             APIs                                                    **/
/*********************************************************************************************************/

/* Motor Init */
void Motor_Init(uint32 Motor_Direction)
{
  /* Startup Variables: apply values from Config Wizard */
  Startup_Config.dutycycle   = 1;                 /* Startup dutycycle from config wizard           */
  Startup_Config.speed_count = Rpm2Ticks( MCTRL_LIB_STARTUP_SPEED_START ); /* Startup speed scaled to system frequency ticks */
  Startup_Config.speed_end   = Rpm2Ticks( MCTRL_LIB_STARTUP_SPEED_END);    /* End of startup speed scaled to system frequency ticks */
  Startup_Config.ramp        = STARTUP_RAMP << 7;                          /* STARTUP_RAMP * 128 */
  
  /* Motor Variables */
  Motor.active_pattern     = 0u;                       /* 0...5 Counter variable PWM Patttern */
  Motor.dutycycle          = Startup_Config.dutycycle; /* Startup Dutycycle 15% */
  Motor.dir                = Motor_Direction;          /* Motor Direction variable */
  Motor.value_60_degree    = 0u;                       /* ticks for 60 degrees */
  Motor.state              = MOTOR_STATE_STARTUP;       /* Startup */
  
  #if (BLDC_HALL_PTN_KNOWN == 0)
    /* set Hall pattern to 0 */
    Hall.patterns[0] = 0;
    Hall.patterns[1] = 0;
    Hall.patterns[2] = 0;
    Hall.patterns[3] = 0;
    Hall.patterns[4] = 0;
    Hall.patterns[5] = 0;
  #else
    /* set Hall pattern to the defines from Config Wizard */    
    Hall.patterns[0] = BLDC_HALL_PTN1;
    Hall.patterns[1] = BLDC_HALL_PTN2;
    Hall.patterns[2] = BLDC_HALL_PTN3;
    Hall.patterns[3] = BLDC_HALL_PTN4;
    Hall.patterns[4] = BLDC_HALL_PTN5;
    Hall.patterns[5] = BLDC_HALL_PTN6;
  #endif

  /* Disable and Clear Hall Interrupts */
  CCU7_disHallEventWrongInt();
  CCU7_clrHallEventWrongIntSts();
  CCU7_disHallEventCorrectInt();
  CCU7_clrHallEventCorrectIntSts();

  /* Enable PWM and Bridge Driver*/
  CCU7_startT12();  /*Start Timer12 */
  BDRV_setBridge(BDRV_chCfg_pwm, BDRV_chCfg_pwm, BDRV_chCfg_pwm, BDRV_chCfg_pwm, BDRV_chCfg_pwm, BDRV_chCfg_pwm); /* Enable Bridge */
}

/* Motor Startup */
void Motor_Startup(void)
{
  COMMUTATION_Timer_Handling(Startup_Config.speed_count); /* Start Commutation Timer */
}

/* Motor Stop */
void Motor_Stop(void)
{
  Motor_State_Set(MOTOR_STATE_STOP);                       /* Motor State Stop */
  Motor_Dutycycle_Set(0u);                                 /* Switch Break Motor (Lowside Mosfets ON) */
  CCU7_setMultiChModePWMPatternShadow(MCTRL_LIB_MCMOUT_Z); /* Z Mode (Motor is running free) */
}

/* Motor Step speed set */
void Motor_Step_Speed_Set(uint32 rpm)
{
  Motor.value_60_degree = Rpm2Ticks(rpm);
}

/* Motor Dutycycle Get */
uint32 Motor_Dutycycle_Get(void)
{
  return Motor.dutycycle;
}

/* Motor Dutycycle Set */
void Motor_Dutycycle_Set(uint32 Dutycycle)
{
  if(Dutycycle <= 1000) /* Limiter */
  {
    Motor.dutycycle = Dutycycle; 
  }
}

/* Motor Dutycycle Set, compensate VDH */
void Motor_VoltageDC_Set(uint32 voltage)
{
  uint16 v_vdh;
  uint32 dutycycle;
  
  v_vdh = 12000;
  // TODO replace ADC1_GetChResult_mV(&v_vdh, ADC1_VDH); /* Reads voltage on VDH PIN */
  dutycycle = (uint32)(1000*voltage)/ (v_vdh+1);  /* calculation ('+1' prevents '0' division) */
  if(dutycycle > 1000)/* Limiter */
  {
    dutycycle = 1000;
  } 
  
  Motor.dutycycle = dutycycle; /* Sets Motor Dutycycle */
}  

/* Motor Speed Get */
uint32 Motor_Speed_Get(void)
{
  return Ticks2Rpm(Motor.value_60_degree);  
} 

/* Motor State Set */
void Motor_State_Set(uint32 state)
{
  Motor.state = state;
}

/* Motor Direction Change, possible Arguments (DIR_BACKWARD / DIR_FORWARD) */
void Motor_Direction_Set(uint32 direction)
{
    Motor.dir = direction; 
}

/* Motor_State_Get */
uint32 Motor_State_Get(void)
{
  return Motor.state;
}

/* Motor Step: Update all MOSFETs Patterns Dutycycles */
void Motor_Step(void)
{ 
  Motor.active_pattern = Update_Dutycycle((uint16)Motor.dutycycle, Motor.active_pattern, Motor.dir);
}  


/*********************************************************************************************************/
/**                                        support functions                                            **/
/*********************************************************************************************************/

/* Scale RPM 2 Timer Ticks*/
uint32 Rpm2Ticks(uint32 rpm)
{
  return( SCU_fCPU_MHz * 1000000 / ( ( ACTIVE_PATTERN_MAX * rpm )/60 ) );
}
/* Scale Timer Ticks 2 RPM */
uint32 Ticks2Rpm(uint32 ticks)
{
  return( SCU_fCPU_MHz * 1000000 / ( ( ACTIVE_PATTERN_MAX * ticks )/60 ) );
}
