/*
 ***********************************************************************************************************************
 *
 * Copyright (c) 2022, Infineon Technologies AG
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 **********************************************************************************************************************/
 
/*******************************************************************************
**                             Author(s) Identity                             **
********************************************************************************
** Initials     Name                                                          **
** ---------------------------------------------------------------------------**
** MK           Martin Kilian                                                 **
** JO           Julia Ott                                                     **
*******************************************************************************/

/*******************************************************************************
**                          Revision Control History                          **
********************************************************************************
** V0.1.0: 2021-05-14, MK:   Initial version of the example (C code, TLE987x) **
** V0.1.1: 2021-10-07, MK:   EP-974: Removed double step in aligmnent,        **
**                           Moved Hall_Handling to #ifdef in WHE ISR         **
** V1.0.0: 2022-09-06, JO:   Ported example from TLE987x to TLE989x           **
*******************************************************************************/

#ifndef BLDC_MOTOR_LIB_H
#define BLDC_MOTOR_LIB_H

/*******************************************************************************
**                                  Includes                                  **
*******************************************************************************/
#include "tle_device.h"
#include "bldc_defines.h"

/*******************************************************************************
**                          Global Macro Declarations                         **
*******************************************************************************/
/* Motor States */
#define MOTOR_STATE_STARTUP 0
#define MOTOR_STATE_RUNNING 20
#define MOTOR_STATE_STOP    30

/* Defines for Motor_Direction_Set() */
#define ACTIVE_PATTERN_MAX  6u /* 0...5  */
#define DIR_FORWARD         1u /* (index + 1u) % 6 => 0 1 2 3 4 5 0... */
#define DIR_BACKWARD        5u /* (index + 5u) % 6 => 0 5 4 3 2 1 0... */

/*******************************************************************************
**                          Global Type Declarations                          **
*******************************************************************************/
/* Motor Variable Structure */
struct Motor
{
  uint32 active_pattern;  /* current Motor pattern */
  uint32 dutycycle;       /* PWM dutycycle, raw value of CCU6 */
  uint32 dir;             /* Motor Direction */
  uint32 value_60_degree; /* Time for 60deg of an electrical round, 1/FSYS ticks */
  uint32 state;           /* Motor States: defined above */
};

/* Startup Variable Structure */
struct Startup_Config
{
  uint32 dutycycle  ; /* dutycylce during startup */
  uint32 speed_end  ; /* ticks at end of startup, initialised with rpm2ticks */
  uint32 speed_count; /* counter variable during startup */
  uint32 ramp       ; /* startup ramp scaled by bitshift --> makes no sense at the moment */
};

/* Hall Variable Structure */
struct Hall
{
  uint32 value_60_degree;
  uint32 speed;
  uint32 delay;
  uint8  patterns[6];
  uint8  padding[2];
};

/*******************************************************************************
**                         Motor Variables                                    **
*******************************************************************************/
extern volatile struct Startup_Config Startup_Config;
extern volatile struct Motor Motor;
extern volatile struct Hall  Hall;


/*******************************************************************************
**                         Global Function Declarations                       **
*******************************************************************************/
void   Motor_Init(uint32 Motor_Direction); 
void   Motor_Startup(void);
void   Motor_Stop(void);
void   Motor_Step_Speed_Set(uint32 rpm);
void   Motor_Dutycycle_Set(uint32 Dutycycle);
uint32 Motor_Dutycycle_Get(void);
void   Motor_VoltageDC_Set(uint32 voltage);
uint32 Motor_State_Get(void);
void   Motor_State_Set(uint32 state);
uint32 Motor_Speed_Get(void);
void   Motor_Direction_Set(uint32 direction);
void   Motor_Step(void);

uint32 Rpm2Ticks(uint32 rpm);
uint32 Ticks2Rpm(uint32 ticks);


#endif
