/*
 ***********************************************************************************************************************
 *
 * Copyright (c) 2021-2023, Infineon Technologies AG
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 **********************************************************************************************************************/

/*******************************************************************************
**                             Author(s) Identity                             **
********************************************************************************
** Initials     Name                                                          **
** ---------------------------------------------------------------------------**
** BG           Blandine Guillot                                              **
** JO           Julia Ott                                                     **
** EE           Erich Englbrecht                                              **
*******************************************************************************/

/*******************************************************************************
**                          Revision Control History                          **
********************************************************************************
** V1.0.0: 2021-04-28, EE:   EP-763: Initial version                          **
** V1.0.1: 2021-05-20, JO:   EP-821: Disable interrupts and trigger SOW before**
**                           calling user_nvm_page_write, close SOW and       **
**                           enable interrupts afterwards                     **
** V1.0.2: 2021-07-27, BG:   EP-814: Added return code for TLE_init()         **
**                           Removed clearing of status registers, should be  **
**                           done in the init functions if they are passed    **
** V1.0.3: 2021-10-06, JO:   EP-949: Corrected memory layout (IROM1 size,     **
**                           IRAM1 start and size, IRAM2 size)                **
** V1.0.4: 2021-10-15, JO:   EP-962: Updated NAC time                         **
** V1.0.5: 2021-11-11, JO:   EP-937: Updated copyright and branding           **
** V1.0.6: 2021-12-20, JO:   EP-977: Added '--diag_suppress 1609' to Asm Misc **
**                           Control to remove compiler warning               **
** V1.1.0: 2022-04-27, JO:   EP-1139: Updated PMU.WD_CTRL.SOW to 1 in         **
**                           Config Wizard                                    **
** V1.2.0: 2023-04-18, JO:   EP-1235: Changed to C startup file               **
** V1.3.0: 2024-06-14, JO:   EP-1480: Disabled  warning 'unsafe buffer usage' **
**                           (ARM compiler for Embedded version >6.21)        **
*******************************************************************************/

/*******************************************************************************
**                                  Abstract                                  **
********************************************************************************
** NVM: Writing a page to the user data flash area                            **
********************************************************************************
** A constant quote is written in the first page of the user data area on     **
** memory bank FLASH0. The written data are read from that page. A checksum   **
** is calculated and checked against the checksum of the initial constant     **
** quote. Success is indicated by LED P0.1 and P0.2, whereas an error is      **
** indicated by LED P1.0 and P1.1.                                            **
*******************************************************************************/

/*******************************************************************************
**                                  Includes                                  **
*******************************************************************************/
#include "bootrom.h"
#include "gpio.h"
#include "tle_device.h"
#include "types.h"
#include "cmsis_misra.h"

/*******************************************************************************
**                                 Constants                                  **
*******************************************************************************/
#define NVM_OPTIONS_NONE (0x00u)


sint32 main(void)
{
  sint8 s8_returnCode;
  uint32 u32_idx;
  uint8 u8_xorChecksum;
  uint8 u8_verifyChecksum;
  sint32 s32_returnValue;
  const uint8 *u8p_userData;
  user_nvm_page_write_t s_pageSource;
  uint8 u8p_dataQuote[UC_FLASH_PAGE_SIZE] = 
  " *********** The best thing about a boolean is even if you are wrong," \
  " you are only off by a bit. [anonymous author] ********** ";

  s_pageSource.data = &u8p_dataQuote[0];
  s_pageSource.nbyte = UC_FLASH_PAGE_SIZE;
  s_pageSource.options = NVM_OPTIONS_NONE;

  /* Clear watchdog fail status */
  PMU_clrFailSafeWatchdogFailSts();

  /* Initialization of hardware modules based on Config Wizard configuration */
  s8_returnCode = TLE_init();
  
  if (s8_returnCode != ERR_LOG_SUCCESS)
  {
    /* Place your code here to handle an initialization error */
  }

  /* Calculate a simple xor checksum for the data quote */
  u8_xorChecksum = 0;
  for (u32_idx = 0; u32_idx < UC_FLASH_PAGE_SIZE; u32_idx++) 
  {
    u8_xorChecksum ^= u8p_dataQuote[u32_idx];
  }

  /* Check if the user data are available for programming the quote */
  if (UC_FLASH0_UDATA_SIZE == 0)
  {
    /* Indicate no available user data by LED P1.2 */
    GPIO_setP12State(GPIO_STATE_HIGH);
  }
  else
  {
    /* suspend and remember all enabled interrupts */
    (void)CMSIS_Irq_Dis();
    /* Open SOW */
    PMU_serviceFailSafeWatchdogSOW();
    /* Write to the first page into the user data area of FLASH0 */
    s32_returnValue = user_nvm_page_write(UC_FLASH0_UDATA_START, &s_pageSource);
    /* Close SOW by regular WDT trigger */
    PMU_serviceFailSafeWatchdog();
    /* reenable suspended interrupts */
    CMSIS_Irq_En();
    	
    if (s32_returnValue == ERR_LOG_SUCCESS) 
    {
      /* Indicate a successful page write by LED P0.1 */
      GPIO_setP01State(GPIO_STATE_HIGH);
    }
    else
    {
      /* Indicate a page write error by LED P1.0 */
      GPIO_setP10State(GPIO_STATE_HIGH);
    }

    /* Set read pointer to first page of the user data area */
    u8p_userData = (uint8 *) UC_FLASH0_UDATA_START;
    
    /* Calculate a simple xor checksum for the data quote */
    u8_verifyChecksum = 0;
    for (u32_idx = 0; u32_idx < UC_FLASH_PAGE_SIZE; u32_idx++)
    {
      u8_verifyChecksum ^= u8p_userData[u32_idx];
    }

    if (u8_verifyChecksum == u8_xorChecksum)
    {
      /* Indicate a successful checksum verification by LED P0.2 */
      GPIO_setP02State(GPIO_STATE_HIGH);
    }
    else
    {
      /* Indicate a wrong checksum verification by LED P1.1 */
      GPIO_setP11State(GPIO_STATE_HIGH);
    }
  }

  for (;;)
  {
    /* Main watchdog service */
    (void) PMU_serviceFailSafeWatchdog();
  }
}
