/*
 ***********************************************************************************************************************
 *
 * Copyright (c) 2021-2023, Infineon Technologies AG
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 **********************************************************************************************************************/

/*******************************************************************************
**                             Author(s) Identity                             **
********************************************************************************
** Initials     Name                                                          **
** ---------------------------------------------------------------------------**
** AA           Abhilash Abhilash                                             **
** CK           Chander Kumar                                                 **
** JO           Julia Ott                                                     **
*******************************************************************************/

/*******************************************************************************
**                          Revision Control History                          **
********************************************************************************
** V1.0.0: 2021-10-15, AA:   EP-961: Initial version                          **
** V1.0.1: 2022-07-05, CK:   EP-961: Added P0.3 for CMAC write fail and       **
**                           LED P0.1 set high only after secure boot         **
**                           successful                                       **
** V1.0.2: 2022-07-27, CK:   EP-961: Fixed warnings/erros for ARMCC v5.06     **
** V1.2.0: 2024-06-14, JO:   EP-1480: Disabled  warning 'unsafe buffer usage' **
**                           (ARM compiler for Embedded version >6.21)        **
*******************************************************************************/

/*******************************************************************************
**                                  Abstract                                  **
********************************************************************************
** SECURE BOOT: Writing the CMAC in the first data page for secure boot       **
** verification                                                               **
********************************************************************************
** Upon first reset, the CMAC for secure boot is generated and written into   **
** the first page.                                                            **
** Upon successful completion, the pin P0.2 is set to high                    **
** The reset is triggered for the second time to check the secure boot.       **
** Upon successful completion, the pin P0.1 is set to high                    **
*******************************************************************************/

/*******************************************************************************
**                                  Includes                                  **
*******************************************************************************/
#include "tle_device.h"
#include "types.h"
#include "gpio.h"
#include "string.h"

/*******************************************************************************
**                                 Constants                                  **
*******************************************************************************/
/** \brief IO buffer size   **/ 
#define PAGE_SIZE (128u)

/** \brief User-defined key slot to be used for encryption, can be anything between 1 and 13, here 2 */
#define SECURE_BOOT_KEY_ID (2u)

/** \brief Secure Boot length, it should be minimal in order to avoid watchdog reset */
#define SECURE_BOOT_LENGTH (0x400u)

/** \brief Flag to check if the CMAC is written in the first page */
#define SECURE_BOOT_CMAC (0x01u)

/** \brief CMAC input buffer length */
#define CMAC_INP_BUFFER_LENGTH (0x20u)

/** \brief CMAC length */ 
#define CMAC_LENGTH (16u)

/** \brief Enabling or disabling the truncation of CMAC length */ 
#define TRUNCATION (0u)

/** \brief Secure Boot start address */
#define SECURE_BOOT_START_ADDRESS (0x11000100u)

/** \brief NVM erase option */
#define NVM_OPTIONS_NONE (0x00u)

/*******************************************************************************
**                                 Variables                                  **
*******************************************************************************/
static uint8 u8_cmacOut[16];  
static uint8 u8_cmacIn[256];
static uint8 u8_flash0Data[PAGE_SIZE];
static user_nvm_page_write_t s_pageWriteData;
static bool b_secureBootCheck = false;

/*******************************************************************************
**                           Function Declarations                            **
*******************************************************************************/
static sint8 writeSecureHashIntoFirstPage(void);


sint32 main(void)
{
  sint8 s8_returnCode;
  
  /* Clear watchdog fail status */
  PMU_clrFailSafeWatchdogFailSts();

  /* Initialization of hardware modules based on Config Wizard configuration */ 
  s8_returnCode = TLE_init();
  
  if (s8_returnCode != ERR_LOG_SUCCESS)
  {
    /* Place your code here to handle an initialization error */
  }
  
  /* Copy first page data  */
  memcpy(&u8_flash0Data[0], (uint8*)UC_FLASH0_START, PAGE_SIZE);
  
  /* 29th Byte is reserved, here used for demonstration purpose */
  if (u8_flash0Data[29] != SECURE_BOOT_CMAC)
  {
    s8_returnCode = writeSecureHashIntoFirstPage();
    
    if (s8_returnCode == ERR_LOG_SUCCESS)
    {
      GPIO_setP02State(GPIO_STATE_HIGH);
    }
    else
    {
      GPIO_setP03State(GPIO_STATE_HIGH);
    }
  }
  else
  {
    b_secureBootCheck = true;
  }
  
  for (;;)
  {
    /* Main watchdog service */
    (void) PMU_serviceFailSafeWatchdog();

    if (b_secureBootCheck)
    {
      GPIO_setP01State(GPIO_STATE_HIGH);
    }
  }
}


/* Perform the CMAC calculation of the user bsl and write into the first page*/
static sint8 writeSecureHashIntoFirstPage(void)
{
  sint8 s8_returnCode;
  user_crypto_io_buf_t s_secureBootCmac;
  user_crypto_cmac_t  s_verifyCmac;
  uint32 u32_cmacOutLength = CMAC_LENGTH;
  uint32 u32_cmacAddress;
 
  /* Initialize the CMAC generation with the specified AES key ID */
  s8_returnCode = (sint8)user_crypto_aes_cmac_generate_start(SECURE_BOOT_KEY_ID);   
  
  if (s8_returnCode == ERR_LOG_SUCCESS)
  {
    uint8 i;
    /* Update the ongoing CMAC generation with the specified buffer */
    s_secureBootCmac.inp.buffer = (uint8*)&u8_cmacIn[0];
    u32_cmacAddress = SECURE_BOOT_START_ADDRESS;  
  
    for (i = 0; i < (SECURE_BOOT_LENGTH/CMAC_INP_BUFFER_LENGTH); i++)
    {     
      memcpy(&u8_cmacIn[0], (uint8*)u32_cmacAddress, CMAC_INP_BUFFER_LENGTH);
      s_secureBootCmac.inp.length = CMAC_INP_BUFFER_LENGTH;
      u32_cmacAddress += CMAC_INP_BUFFER_LENGTH;
      
      s8_returnCode = (sint8)user_crypto_aes_cmac_generate_update(&s_secureBootCmac.inp);
    }
  }
  
  /* Finalize the ongoing CMAC generation with the specified input/output buffer */
  s_secureBootCmac.out.buffer = (uint8*)&u8_cmacOut[0];
  s_secureBootCmac.out.length = &u32_cmacOutLength;
  s_secureBootCmac.inp.length = 0; /* Reset the input length */
  s8_returnCode |= (sint8)user_crypto_aes_cmac_generate_finish(&s_secureBootCmac, TRUNCATION);

  if (s8_returnCode == ERR_LOG_SUCCESS)
  {
    /* Initializes the CMAC verification with the specified AES key ID */ 
    s8_returnCode = (sint8)user_crypto_aes_cmac_verify_start(SECURE_BOOT_KEY_ID);
    
    if (s8_returnCode == ERR_LOG_SUCCESS)
    {
      uint8 i;
      /* Direct address pointer cannot be assigned, otherwise it results in an error */
      s_verifyCmac.inp.buffer = (uint8*)&u8_cmacIn[0];
      u32_cmacAddress = SECURE_BOOT_START_ADDRESS;
      
      /* 1024 bytes must be encrypted; given that the update function can encrypt 32 bytes at a time, it must be performed 32 times */
      for (i = 0; i < (SECURE_BOOT_LENGTH/CMAC_INP_BUFFER_LENGTH); i++)
      {     
        memcpy(&u8_cmacIn[0], (uint8*)u32_cmacAddress, 0x80);
        s_verifyCmac.inp.length = CMAC_INP_BUFFER_LENGTH;
        u32_cmacAddress += CMAC_INP_BUFFER_LENGTH;      
        /* Updates an ongoing CMAC verification with the specified input buffer */  
        s8_returnCode = (sint8)user_crypto_aes_cmac_verify_update(&s_verifyCmac.inp);
      }
    }
    
    /* Reset the input length*/
    s_verifyCmac.inp.length = 0;
    s_verifyCmac.mac.buffer = (uint8*)&u8_cmacOut[0];
    s_verifyCmac.mac.length = u32_cmacOutLength;
    /*Finalizes an ongoing CMAC verification with the specified MAC */
    s8_returnCode |= (sint8)user_crypto_aes_cmac_verify_finish(&s_verifyCmac);
  }
  
  /* Copy CMAC to first data page */
  memcpy(&u8_flash0Data[8], &u8_cmacOut[0], CMAC_LENGTH);
  /* Copy length of the code to first data page */
  u8_flash0Data[24] = 0x00;
  u8_flash0Data[25] = 0x04;
  u8_flash0Data[26] = 0x00;
  u8_flash0Data[27] = 0x00;
  u8_flash0Data[28] = SECURE_BOOT_KEY_ID;
  /* Set the flag indicating that CMAC is written */
  u8_flash0Data[29] = SECURE_BOOT_CMAC; 
  
  if (s8_returnCode == ERR_LOG_SUCCESS)
  { 
    /* Erase the first page  */
    s8_returnCode = (sint8)user_nvm_page_erase(UC_FLASH0_START, NVM_OPTIONS_NONE);
  
    if (s8_returnCode == ERR_LOG_SUCCESS)
    { 
      s_pageWriteData.data = (uint8*)&u8_flash0Data;
      s_pageWriteData.nbyte = PAGE_SIZE;
      s_pageWriteData.options = NVM_OPTIONS_NONE; 
      /* Write data into first page */
      s8_returnCode = (sint8)user_nvm_page_write(UC_FLASH0_START, &s_pageWriteData);
    }
  }
  
  return s8_returnCode;
}



