/*
 ***********************************************************************************************************************
 *
 * Copyright (c) Infineon Technologies AG
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 **********************************************************************************************************************/

/*******************************************************************************
**                                  Includes                                  **
*******************************************************************************/
#include <stdio.h>
#include "security_access.h"

/*******************************************************************************
**                         Private Macro Declarations                         **
*******************************************************************************/

/**\brief Global IO structure declaration */
static user_crypto_io_buf_t s_inpMsg;

/**\brief Array to store encrypted data*/
static uint8 u8_buf[TLE989X_AES_IO_BUFFER_SIZE]; 

/**\brief Flag which monitors seed request*/
static bool b_seedReceived = false;

/*******************************************************************************
**                        Private Function Declarations                       **
*******************************************************************************/

static sint8 execAES(user_crypto_io_buf_t *sp_aesIO, user_crypto_fid_t fid, uint32 u32_keyId);
static void sendNRC(tDiagMsg *sp_msg, uint8 u8_nrc);

/*******************************************************************************
**                         Private Function Definitions                       **
*******************************************************************************/

/** \brief Encrypt 32-byte input data
 *
 * \param sp_aesIO, pointer to input message 
 * \param e_fid, ID of the desired operation
 * \param u32_keyId, secure key ID, key ID range is 0–13
 * \return sint32 0: success, <0: error codes
 */
static sint8 execAES(user_crypto_io_buf_t *sp_aesIO, user_crypto_fid_t e_fid, uint32 u32_keyId)
{
  sint8 s8_returnCode;
  uint32 u32_encodeOutLength;
  uint32 u32_totalLength= 0;
  
  /* Initialize a ECB encryption operation */
  s8_returnCode = (sint8)user_crypto_aes_start(e_fid, u32_keyId, NULL_PTR);
  
  /* Update the ongoing ECB encryption operation */
  if (s8_returnCode == ERR_LOG_SUCCESS)
  {
    s8_returnCode = (sint8)user_crypto_aes_update(sp_aesIO);
    u32_totalLength += *(sp_aesIO->out.length);
    u32_encodeOutLength = REQUEST_LENGTH - 2;   
    
    sp_aesIO->inp.buffer = sp_aesIO->inp.buffer + AES_UPDATE_LENGTH;
    /* Reset the input length */
    sp_aesIO->inp.length = 0;   
    sp_aesIO->out.buffer = (uint8 *)(sp_aesIO->out.buffer + u32_totalLength);
    sp_aesIO->out.length = &u32_encodeOutLength;
    
    /* Finalize the ongoing ECB encryption operation */
    s8_returnCode = (sint8)user_crypto_aes_finish(sp_aesIO);
  }
  else
  {
    /* Finalize the ongoing ECB encryption operation */
    s8_returnCode = (sint8)user_crypto_aes_finish(sp_aesIO);
  }
   
  return s8_returnCode;
}

/** \brief Send a NRC (Negative Response Code), format: 7F|Service ID|NRC CODE
 */
static void sendNRC(tDiagMsg *sp_msg, uint8 u8_nrc)
{
  sp_msg->u8_resp[0]     = NEG_RESP_ID;
  sp_msg->u8_resp[1]     = SECURITY_ACCESS_REQ_SERVICE_ID;
  sp_msg->u8_resp[2]     = u8_nrc;
  sp_msg->u32_respLength = NEG_RESP_LENGTH;
}

/*******************************************************************************
**                         Global Function Definitions                        **
*******************************************************************************/

/**\brief This function uses ISO UDS Diagnostic protocol. Seed and key length is 32 bytes.
 * \brief Step 1: The diagnostic tool should send data as follows: 2-byte (Service ID(0x27) and Subfunction(0x01)) + 32-byte seed
 * \brief Step 2: The device shall send +ve/-ve response.
 * \brief Step 3: The diagnostic tool should send data as follows: 2-byte(Service ID(0x27) and Subfunction(0x02)), KEY = AES_ENC(SEED) + 32-byte key
 * \brief Step 4: If the received key is correct, then the device sends +ve response
 *
 * \param Input : Pointer to the diagnostic request message 
 * \param Output: +ve or -ve response depending on the message
 */
void serviceSecurityAccess(tDiagMsg *sp_msg)
{
  uint32 u32_outputLength;
  uint8 i;
  
  /*  Minimum output length must be provided, otherwise encryption fails */
  u32_outputLength = REQUEST_LENGTH - 2;  
  
  if (sp_msg->u32_reqLength == REQUEST_LENGTH)
  {
    if (sp_msg->u8_req[1] == SEED_REQUEST)
    {
      s_inpMsg.inp.buffer = (uint8*)&sp_msg->u8_req[2];
      s_inpMsg.inp.length = REQUEST_LENGTH - 2;
      s_inpMsg.out.buffer = (uint8*)&u8_buf[0];
      s_inpMsg.out.length = (uint32*)&u32_outputLength;
      
      /* ECB encryption is used in this example */
      if (execAES(&s_inpMsg, CRYPTO_ECB_ENCRYPT, KEY_ID) == ERR_LOG_SUCCESS)
      {
        /* Send positive response */
        sp_msg->u8_resp[0]     = POS_RESP_ID;
        sp_msg->u8_resp[1]     = SEED_REQUEST;
        sp_msg->u32_respLength = POS_RESP_LENGTH;
        b_seedReceived = true;
      }
      else
      {
        /* Encryption failed */
        sendNRC(sp_msg, WRONG_KEY);
      }
    }
    else if ((sp_msg->u8_req[1] == KEY_REQUEST) && (b_seedReceived == true))
    {
      for (i = 2; i < REQUEST_LENGTH; i++)
      {
        if (sp_msg->u8_req[i] != u8_buf[i-2])
        {
          /* Wrong key */
          sendNRC(sp_msg, WRONG_KEY);
          return;
        }      
      }
      /* Received key is correct, send positive response */
      sp_msg->u8_resp[0]     = POS_RESP_ID;
      sp_msg->u8_resp[1]     = KEY_REQUEST;
      sp_msg->u32_respLength = POS_RESP_LENGTH;
      b_seedReceived = false;
    }
    else if ((sp_msg->u8_req[1] == KEY_REQUEST) && (b_seedReceived == false))
    {
      /* Seed request not received, sequence error */
      sendNRC(sp_msg, SEQUENCE_ERROR);
    }
    else
    {
      /* Subfunction not supported */
      sendNRC(sp_msg, SUBFUNC_ERROR);
    }
  }
  else
  {
    /* Incorrect length */
    sendNRC(sp_msg, LENGTH_ERROR);
  }
}


