/*
 ***********************************************************************************************************************
 *
 * Copyright (c) 2020-2023, Infineon Technologies AG
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 **********************************************************************************************************************/

/*******************************************************************************
**                             Author(s) Identity                             **
********************************************************************************
** Initials     Name                                                          **
** ---------------------------------------------------------------------------**
** AA           Abhilash Abhilash                                             **
** CK           Chander Kumar                                                 **
** JO           Julia Ott                                                     **
*******************************************************************************/

/*******************************************************************************
**                          Revision Control History                          **
********************************************************************************
** V1.0.0: 2020-10-21, AA:   EP-924: Initial version                          **
** V1.0.1: 2022-05-25, CK:   EP-924: Modified chartoHex function              **
** V1.0.2: 2022-07-01, CK:   EP-924: Removed echo back for received byte      **
**                           to terminal and added echo of complete received  **
**                           string in main loop                              **
** V1.1.0: 2023-04-18, JO:   EP-1235: Changed to C startup file               **
** V1.2.0: 2024-06-14, JO:   EP-1480: Disabled  warning 'unsafe buffer usage' **
**                           (ARM compiler for Embedded version >6.21)        **
*******************************************************************************/

/*******************************************************************************
**                                  Abstract                                  **
********************************************************************************
** SECURITY: Computes CMAC and verifying the data received via UART           **
********************************************************************************
** When starting this example a command line menu is printed on the console   **
** using the UART virtual COM port of J-Link. The implemented command line    **
** handler offers two simple request message format to show the computation   **
** of CMAC in the eval device.                                                **
** Step 1: The 16-Byte input request(SEED) must be sent using UART            **
** Step 2: The device performs the encryption and stores in the buffer        **
** Step 3: The CMAC(KEY) is sent for verification                             **
** Step 4: The device checks if the received data are correct, then sends     ** 
**         either positive OR negative response                               **
*******************************************************************************/

/*******************************************************************************
**                                  Includes                                  **
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "tle_device.h"
#include "types.h"
#include "security_access.h"

/*******************************************************************************
**                                 Constants                                  **
*******************************************************************************/
#define BUFFER_SIZE (36U)

/*******************************************************************************
**                                 Variables                                  **
*******************************************************************************/
static volatile bool b_cmdTrigger = true;
static volatile uint8 u8_readCnt = 0;
static uint8 u8_buffer[BUFFER_SIZE];
static tDiagMsg s_uartMsg;

/*******************************************************************************
**                           Function Declarations                            **
*******************************************************************************/
void handleCmd(void);
void charToHex(uint8 *u8p_charIn, uint8 * u8p_byteOut);


sint32 main(void)
{
  sint8 s8_returnCode;
  uint8 i;
  
  /* Clear watchdog fail status */
  PMU_clrFailSafeWatchdogFailSts();
  
  /* Initialization of hardware modules based on Config Wizard configuration */
  s8_returnCode = TLE_init();
  
  if (s8_returnCode != ERR_LOG_SUCCESS)
  {
  /* Place your code here to handle an initialization error */
  }

  for (;;)
  {
    /* Main watchdog service */
    (void) PMU_serviceFailSafeWatchdog();

    /* Check for activated command handler trigger */
    if (b_cmdTrigger == true)
    {
      /* Echo access command received via UART */
      for(i = 0; i<BUFFER_SIZE; i++)
      {
        printf("%c",u8_buffer[i]);
      }
      /* Handle new command */
      handleCmd();
    }
  }
}


/* Callback for UART reception */
void uart_receive()
{
  /* Check for buffer overflow */
  if (u8_readCnt < BUFFER_SIZE)
  {
    /* Get byte from P1.2 */
    u8_buffer[u8_readCnt] = (uint8)stdin_getchar();
    /* Check if received character was a newline */
    if (u8_buffer[u8_readCnt] == 0x0d)
    {
      charToHex(&u8_buffer[0],&s_uartMsg.u8_req[0]);
      /* Remove the additional 1 byte length  */
      u8_readCnt = u8_readCnt -1;
      /* Trigger command handler */
      b_cmdTrigger = true;
    }
    /* Increment receive counter for new character */
    u8_readCnt++;
  }
  else
  {
    /* Convert to byte data */
      charToHex(&u8_buffer[0],&s_uartMsg.u8_req[0]);
    /* Trigger command handler */
    b_cmdTrigger = true;
  }
}


/* Command handler */
void handleCmd()
{
  uint8 i;
  
  /* Check for initial call - only the menu must be printed */
  if (u8_readCnt != 0)
  {
    s_uartMsg.u32_reqLength = u8_readCnt/2;
    if (s_uartMsg.u8_req[0] == SECURITY_ACCESS_REQ_SERVICE_ID)
    {     
      serviceSecurityAccess(&s_uartMsg);
    }
    else
    {
      s_uartMsg.u8_resp[0]     = NEG_RESP_ID;
      s_uartMsg.u8_resp[1]     = SECURITY_ACCESS_REQ_SERVICE_ID;
      s_uartMsg.u8_resp[2]     = SERVICE_NOT_SUPPORTED;
      s_uartMsg.u32_respLength = NEG_RESP_LENGTH;
    }
    
    /* Send out response via stdout */
    printf("\n --> Response 0x%x", s_uartMsg.u8_resp[0]);
    for(i = 1; i< s_uartMsg.u32_respLength; i ++)
    {
      printf(" 0x%x", s_uartMsg.u8_resp[i]);
    }
    printf("\n");
  }

  /* Print a fancy simple command menu */
  printf("\n***  Message request menu  ***\n");
  printf(" Send request 27|01|16-byte seed| \n");
  printf(" Send request 27|02|16-byte key| \n");
  printf("Enter your request:> ");

  /* Reset u8_readCnt for new command */
  u8_readCnt = 0;
  /* Deactivate command handler until new one is received */
  b_cmdTrigger = false;
}


/* Convert char data to hex data */
void charToHex(uint8 *u8p_charIn, uint8 * u8p_byteOut)
{
  uint8 u8_var[BUFFER_SIZE];
  uint8 i;
  uint8 j;
  
  for (i = 0; i< BUFFER_SIZE; i++)
  {
    /* From 0x0 to 0x9 */
    if(u8p_charIn[i] >= 48  && u8p_charIn[i] < 64)
    {
      u8_var[i] = u8p_charIn[i] - 48;
    }
    /* From 0xa to 0xf */
    else if (u8p_charIn[i] >= 65  && u8p_charIn[i] <= 70)
    {
      u8_var[i] = u8p_charIn[i] - 55;
    }
    /* From 0xA to 0xF */
    else if (u8p_charIn[i] >= 97  && u8p_charIn[i] <= 102)
    {
      u8_var[i] = u8p_charIn[i] - 87;
    }
    /* Carriage return*/
    else if (u8p_charIn[i] == 0x0D )
    {
      u8_var[i] = 0;
    }
    /* default calue for all other cases */
    else
    {
      u8_var[i] = 0;
    }
  }
  
  for (j = 0; j<(BUFFER_SIZE); j +=2)
  {
    u8p_byteOut[j/2] = (uint8)((u8_var[j] & 0x0F)<<4) | (u8_var[j + 1] & 0x0F);
  }
}
