/*
 ***********************************************************************************************************************
 *
 * Copyright (c) Infineon Technologies AG
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 **********************************************************************************************************************/

/*******************************************************************************
**                                  Includes                                  **
*******************************************************************************/
#include <stdio.h>
#include "security_access.h"

/*******************************************************************************
**                         Private Macro Declarations                         **
*******************************************************************************/

/**\brief Global IO structure declaration */
static user_crypto_io_buf_t s_inpMsg;

/**\brief Array to store encrypted data */
static uint8 u8_buf[TLE989X_AES_IO_BUFFER_SIZE]; 

/**\brief Flag which monitors seed request */
static bool b_seedReceived = false;

/*******************************************************************************
**                        Private Function Declarations                       **
*******************************************************************************/

static sint8 execCMAC(uint8 *u8p_dataIn, uint32 u32_length, uint32 u32_keyId, bool b_truncation);
static void sendNRC (tDiagMsg *sp_msg, uint8 u8_nrc);

/*******************************************************************************
**                         Private Function Definitions                       **
*******************************************************************************/

/** \brief  Generate 16-byte CMAC 
 *
 * \param u8p_dataIn, pointer to the input array of data
 * \param u32_length, data length
 * \param u32_keyId, secure key ID, key ID range is 1 to 13
 * \param b_truncation, boolean variable which enables or disables the truncation of CMAC
 * \return sint32 0: success, <0: error codes
 */
static sint8 execCMAC(uint8 *u8p_dataIn, uint32 u32_length, uint32 u32_keyId, bool b_truncation)
{
  sint8 s8_returnCode;
  user_crypto_cmac_t s_verifyCmac;
  
  /* Minimum output length must be provided, otherwise encryption fails */
  uint32 u32_outLength = CMAC_LENGTH; 
  
  s_inpMsg.inp.buffer = (uint8*)&u8p_dataIn[0];
  s_inpMsg.inp.length = u32_length ;
  s_inpMsg.out.buffer = (uint8*)&u8_buf;
  s_inpMsg.out.length = &u32_outLength;
  
  /* Initialize CMAC operation */
  s8_returnCode = (sint8)user_crypto_aes_cmac_generate_start(u32_keyId);
  
  if (s8_returnCode == ERR_LOG_SUCCESS)
  {
    /* Update the ongoing CMAC generation with the specified buffer */
    s8_returnCode = (sint8)user_crypto_aes_cmac_generate_update(&s_inpMsg.inp);    
  }
	
  /* Reset the input length  */
  s_inpMsg.inp.length = 0; 
  s8_returnCode = (sint8)user_crypto_aes_cmac_generate_finish(&s_inpMsg, b_truncation);
  
  /* Initialize the CMAC verification with the specified AES key ID */
  s8_returnCode |= (sint8)user_crypto_aes_cmac_verify_start(u32_keyId);
  
	if (s8_returnCode == ERR_LOG_SUCCESS)
  {
		s_verifyCmac.inp.buffer = (uint8*)&u8p_dataIn[0];
    s_verifyCmac.inp.length =  u32_length ;
    
    /* Update the ongoing CMAC verification with the specified input buffer */
    s8_returnCode = (sint8)user_crypto_aes_cmac_verify_update(&s_verifyCmac.inp);
  }
	
  /* Reset the input length */
  s_verifyCmac.inp.length = 0;
  s_verifyCmac.mac.buffer = (uint8*)&u8_buf;
  s_verifyCmac.mac.length =  CMAC_LENGTH;
  
  /* Finalize the ongoing CMAC verification */
  s8_returnCode = (sint8)user_crypto_aes_cmac_verify_finish(&s_verifyCmac);

  return s8_returnCode;
}

/** \brief Send a NRC (Negative Response Code), format: 7F|Service ID|NRC CODE
 */
static void sendNRC(tDiagMsg *sp_msg, uint8 u8_nrc)
{
  sp_msg->u8_resp[0]     = NEG_RESP_ID;
  sp_msg->u8_resp[1]     = SECURITY_ACCESS_REQ_SERVICE_ID;
  sp_msg->u8_resp[2]     = u8_nrc;
  sp_msg->u32_respLength = NEG_RESP_LENGTH;
}

/*******************************************************************************
**                         Global Function Definitions                        **
*******************************************************************************/

/** \brief This function uses ISO UDS diagnostic protocol. Seed and key length is 16 bytes.
 * \brief Step 1: The diagnostic tool should send data as follows: 2-byte data (Service ID(0x27) and Subfunction(0x01)) + 16-byte seed
 * \brief Step 2: The device shall send +ve/-ve response
 * \brief Step 3: The diagnostic tool should send data as follows: 2-byte data (Service ID(0x27) and Subfunction(0x02)), KEY = CMAC(SEED) + 16-byte key
 * \brief Step 4: If the received key is correct, then the device sends +ve response
 *
 * \param Input : Pointer to the diagnostic request message
 * \param Output: +ve or -ve response depending on the message
 */
void serviceSecurityAccess(tDiagMsg *sp_msg)
{
  uint8 i;

  if (sp_msg->u8_req[1] == SEED_REQUEST)
  {
    /* Check exact length */     
    if (sp_msg->u32_reqLength == REQUEST_LENGTH)
    {
      /* Execute CMAC operation */
      if (execCMAC(&sp_msg->u8_req[2], (REQUEST_LENGTH -2), KEY_ID, false)== ERR_LOG_SUCCESS)
      {
        /* Send positive response */
        sp_msg->u8_resp[0]     = POS_RESP_ID;
        sp_msg->u8_resp[1]     = SEED_REQUEST;
        sp_msg->u32_respLength = POS_RESP_LENGTH;
        b_seedReceived = true;
      }
      else
      { 
        /* CMAC generation failed */       
        sendNRC(sp_msg, WRONG_KEY);  
      }
    }
    else
    { 
      /* Incorrect length */
      sendNRC(sp_msg, LENGTH_ERROR); 
    }   
  }
  else if ((sp_msg->u8_req[1] == KEY_REQUEST) && (b_seedReceived == true))
  {   
    /* Check exact length */     
    if (sp_msg->u32_reqLength == REQUEST_LENGTH)
    {
      for( i = 2; i < REQUEST_LENGTH; i++)
      {
        if (sp_msg->u8_req[i] != u8_buf[i-2])
        {
          /* Wrong key */
          sendNRC(sp_msg, WRONG_KEY);  
          return;
        }      
      }
      
      /* Correct key received, send positive response */
      sp_msg->u8_resp[0]     = POS_RESP_ID;
      sp_msg->u8_resp[1]     = KEY_REQUEST;
      sp_msg->u32_respLength = POS_RESP_LENGTH;
      b_seedReceived = false;
    }
    else
    { 
      /* Incorrect length */      
      sendNRC(sp_msg, LENGTH_ERROR); 
    } 
  }
  else if ((sp_msg->u8_req[1] == KEY_REQUEST) && (b_seedReceived == false))
  {
    /* Sequence error */
    sendNRC(sp_msg, SEQUENCE_ERROR); 
  }
  else
  {  
    /* Subfunction not supported */   
    sendNRC(sp_msg, SUBFUNC_ERROR); 
  }
}

