/*
 ***********************************************************************************************************************
 *
 * Copyright (c) Infineon Technologies AG
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 **********************************************************************************************************************/

/*******************************************************************************
**                                  Includes                                  **
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "keywrite.h"

/*******************************************************************************
**                         Private Function Declarations                      **
*******************************************************************************/

static sint8 execCMAC(uint16 u16_keyId, uint8 *u8p_inpBuf, uint32 u32_inpBufLength, uint8 *u8p_outBuf, uint32 *u32p_outBufLength);
static sint8 execAES(uint16 u16_keyId, user_key_t * s_key, uint32 u32_keyLength, uint8 *u8p_encryptKeyBuf, uint32 *u8p_encryptKeyBufLength);

/*******************************************************************************
**                         Private Function Definitions                       **
*******************************************************************************/

/** \brief Execute CMAC operation
 *
 * \param u32_keyId, secure key ID, key ID range is 1 to 13
 * \param u8p_inpBuf, pointer to input data 
 * \param u32_inpBufLength, input data length
 * \param u8p_outBuf, pointer to output data 
 * \param u32p_outBufLength, pointer output data length
 * \return sint32 0: success, <0: error codes
 */
static sint8 execCMAC(uint16 u16_keyId, uint8 *u8p_inpBuf, uint32 u32_inpBufLength, uint8 *u8p_outBuf, uint32 *u32p_outBufLength)
{
  sint8 s8_returnCode;
  user_crypto_inp_buf_t s_cmacInp;
  user_crypto_io_buf_t s_cmacIO;
  uint32 u32_cmacOutLength;

  /* Initialize the CMAC generation with the specified AES key ID */
  s8_returnCode = (sint8)user_crypto_aes_cmac_generate_start(u16_keyId);

  /* Update the ongoing CMAC generation with the specified buffer */
  s_cmacInp.buffer = u8p_inpBuf;
  s_cmacInp.length = u32_inpBufLength;
  
  if (s8_returnCode == ERR_LOG_SUCCESS)
  {
    s8_returnCode = (sint8)user_crypto_aes_cmac_generate_update(&s_cmacInp);
  }

  /* Finalize the ongoing CMAC generation with the specified input/output buffer */
  u32_cmacOutLength = CMAC_LENGTH;
  s_cmacIO.inp.buffer = s_cmacInp.buffer;
  s_cmacIO.inp.length = 0; /* Reset the input length to zero */
  s_cmacIO.out.buffer = u8p_outBuf;
  s_cmacIO.out.length = &u32_cmacOutLength;

  s8_returnCode |= (sint8)user_crypto_aes_cmac_generate_finish(&s_cmacIO, TRUNCATION);

  *u32p_outBufLength = u32_cmacOutLength;

  return s8_returnCode;
}

/** \brief Execute input key CBC-encryption operation 
 *
 * \param u32_keyId, secure key ID, key ID range is 1 to 13
 * \param sp_key,  pointer to input key structure 
 * \param u32_keyLength input key length
 * \param u8p_encryptKeyBuf  pointer to output data 
 * \param u8p_encryptKeyBufLength output data length
 * \return sint32 0: success, <0: error codes
 */
static sint8 execAES(uint16 u16_keyId, user_key_t *sp_key, uint32 u32_keyLength, uint8 *u8p_encryptKeyBuf, uint32 *u8p_encryptKeyBufLength)
{
  sint8 s8_returnCode;
	
	/* Initialization vector to be used for CBC mode AES encryption */
  uint8 u8_iv[IV_LENGTH] = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15};

  user_crypto_cbc_t s_cryptoCBC;
  user_crypto_io_buf_t s_aesIO;

  uint32 u32_totalEncryptKeyBufLength = 0;  

  s_cryptoCBC.iv = u8_iv;
  s_cryptoCBC.iv_length = IV_LENGTH;

  /* Initialize a CBC encryption operation */
  s8_returnCode = (sint8)user_crypto_aes_start(CRYPTO_CBC_ENCRYPT, u16_keyId, &s_cryptoCBC);
  
  if (s8_returnCode == ERR_LOG_SUCCESS)
  {
    s_aesIO.inp.buffer = (uint8*)sp_key;
    s_aesIO.inp.length = u32_keyLength;
    s_aesIO.out.buffer = u8p_encryptKeyBuf;
    s_aesIO.out.length = u8p_encryptKeyBufLength;
       
    /* Update the CBC encryption operation */
    s8_returnCode = (sint8)user_crypto_aes_update(&s_aesIO);

    u32_totalEncryptKeyBufLength += *u8p_encryptKeyBufLength;
  }
  
  /* Reset the input length to zero */
  s_aesIO.inp.length = 0;
  s_aesIO.out.buffer = (uint8_t *)(u8p_encryptKeyBuf + *u8p_encryptKeyBufLength);
  
  /* Finalize the ongoing CBC encrypted operation */
  s8_returnCode |= (sint8)user_crypto_aes_finish(&s_aesIO);

  u32_totalEncryptKeyBufLength += *u8p_encryptKeyBufLength;
  *u8p_encryptKeyBufLength = u32_totalEncryptKeyBufLength;
  
  return s8_returnCode;
}


/*******************************************************************************
**                         Global Function Definitions                        **
*******************************************************************************/

/** \brief Perform key encryption following CMAC generation and write key into the requested slot
 *
 * \param u32_keyId, secure key ID, key ID range is 1 to 13
 * \param inpKey, pointer to input key structure 
 * \return sint32 0: success, <0: error codes
 */
sint8 writeUserKey(user_key_t *sp_inpKey, uint16 u16_keyId)
{
  user_key_t s_key;
  user_key_write_t s_keyWrite;
  volatile sint8 s8_returnCode;
  
  uint32 u8p_encryptKeyBufLength = TLE989X_BUFFER_SIZE;
  uint8 u8p_encryptKeyBuf[TLE989X_BUFFER_SIZE];
  uint32 u32_cmacOutLength = TLE989X_BUFFER_SIZE;
  
	/* Copy the encypted user key to the key write params  */	
  memcpy(&s_key, sp_inpKey, sizeof(s_key));

  /* The input key, version, length and protection byte need to be encrypted using CBC mode */             
  s8_returnCode = execAES(u16_keyId, &s_key, sizeof(s_key), u8p_encryptKeyBuf, &u8p_encryptKeyBufLength);
  
  if (s8_returnCode == ERR_LOG_SUCCESS)
  {             
    memcpy(s_keyWrite.params.encrypted_key_buf, u8p_encryptKeyBuf, u8p_encryptKeyBufLength);
		/* Update the target key slot id and the key slot id used for encryption */
    s_keyWrite.params.target_key_id  = u16_keyId;
    s_keyWrite.params.encrypt_key_id = u16_keyId;
    
    /* CMAC calculation for the encrypted buffer, target slot id and encrypt slot id */           
    s8_returnCode = execCMAC(u16_keyId, (uint8*)&s_keyWrite.params, sizeof(s_keyWrite.params), s_keyWrite.signature, &u32_cmacOutLength);
    
    if (s8_returnCode == ERR_LOG_SUCCESS)
    {
			/* This user API function writes a cryptographic key */
      s8_returnCode =  (sint8)user_crypto_key_write(&s_keyWrite);
    }
  }
  
  return s8_returnCode;
}
