/*
 ***********************************************************************************************************************
 *
 * Copyright (c) 2020-2023, Infineon Technologies AG
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 **********************************************************************************************************************/

/*******************************************************************************
**                             Author(s) Identity                             **
********************************************************************************
** Initials     Name                                                          **
** ---------------------------------------------------------------------------**
** AA           Abhilash Abhilash                                             **
** CK           Chander Kumar                                                 **
** JO           Julia Ott                                                     **
*******************************************************************************/

/*******************************************************************************
**                          Revision Control History                          **
********************************************************************************
** V1.0.0: 2020-10-21, AA:   EP-925: Initial version                          **
** V1.0.1: 2022-07-07, CK:   EP-925: Added UART communication to get the      **
**                           key slot id and version, addded function to      **
**                           convert char to hex/int                          **
** V1.1.0: 2023-04-18, JO:   EP-1235: Changed to C startup file               **
** V1.2.0: 2024-06-14, JO:   EP-1480: Disabled  warning 'unsafe buffer usage' **
**                           (ARM compiler for Embedded version >6.21)        **
*******************************************************************************/

/*******************************************************************************
**                                  Abstract                                  **
********************************************************************************
** SECURITY : Writing the new user key into the requested slot                **
********************************************************************************
** The device performs the necessary computation and writes the new key       **
** in the requested slot. In case of success, the pin P0.1 is set, otherwise  **
** the pin P0.2 is set.                                                       **
*******************************************************************************/

/*******************************************************************************
**                                  Includes                                  **
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "tle_device.h"
#include "types.h"
#include "keywrite.h"

/*******************************************************************************
**                                 Constants                                  **
*******************************************************************************/
/** \brief Key protection, 0xFF indicates the write protection is disabled */
#define KEY_PROTECTION (0xFF)

/** \brief Length of the user key  */
#define KEY_LENGTH (0x20)

/*******************************************************************************
**                                 Variables                                  **
*******************************************************************************/
static uint8 u8_rawKey[KEY_LENGTH] = {0xA1, 0xA2, 0xA3, 0xA4, 0xA5, 0xA6, 0xA7, 0xA8, 0xA9, 0xAA, 0xAB, 0xAC, 0xAD, 0xAE, 0xAF, 0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x1A, 0x1B, 0x1C, 0x1D, 0x1E, 0x1F, 0x20};

/*******************************************************************************
**                           Function Declarations                            **
*******************************************************************************/
uint8 charToHex(uint8 charX);


sint32 main(void)
{
  /* Local variable declaration */
  user_key_t s_key;
  sint8 s8_returnCode;
  
  uint8 u8_slotID = 0;
  uint16 u16_keyVersion = 0;
  uint8 c_recvdChar = 0x0;
  bool b_validKeyVersionRecvd = false;
  bool b_validSlotIDRecvd = false;

  /* Clear watchdog fail status */
  PMU_clrFailSafeWatchdogFailSts();

  /* Initialization of hardware modules based on Config Wizard configuration */ 
  s8_returnCode = TLE_init();
  
  if (s8_returnCode != ERR_LOG_SUCCESS)
  {
    /* Place your code here to handle an initialization error */
  }
  
  /* Print message to get the Slot ID */
  printf("\nEnter the Slot ID in range 1-12:");
  
  /* Wait in loop to get the slod it from user(UART terminal)*/
  while(c_recvdChar != 0x0d)
  {
    c_recvdChar = (uint8)stdin_getchar();
    if (c_recvdChar != 0x0d)
    {
      u8_slotID = u8_slotID*10 + (uint8)charToHex(c_recvdChar);
    }
  }
	/* Echo received slot ID on UART console */
	printf("%d",u8_slotID);
  
  /* Check if the recived Slod ID is in valid range */
  if(u8_slotID >= 1 && u8_slotID <= 12){
    b_validSlotIDRecvd = true; 
  }
  else
  {
   printf("\nSlot ID out of range\n");
  }
  
  if(b_validSlotIDRecvd)
  {
    /* Print message to get the Key version */
    printf("\nEnter the Key Version in range 0000 - FFFF :");
  
    c_recvdChar = 0x0;
    
    /* Wait in loop to get the slod it from user(UART terminal)*/
    while(c_recvdChar != 0x0d)
    {
      c_recvdChar = (uint8)stdin_getchar();
      if (c_recvdChar != 0x0d)
      {
        u16_keyVersion = u16_keyVersion*16 + (uint8)charToHex(c_recvdChar);
      }
    }
		
		/* Echo received key version on UART console */
		printf("%x",u16_keyVersion);
		
		/* Check if Key version is in valid range */
    if(u16_keyVersion > 0x0000 && u16_keyVersion <= 0xFFFE)
    {
      b_validKeyVersionRecvd = true; 
    }
    else
    {
      printf("\nKey version out of range\n");
    }
  }
  
  if(b_validKeyVersionRecvd)
  {
    /* Configure the key parameter*/
    s_key.protection = KEY_PROTECTION;
    s_key.length     = KEY_LENGTH;
    s_key.version    = u16_keyVersion;
    
    /* Copy the new key into user_key_t structure */ 
    memcpy(&s_key.key, &u8_rawKey[0], KEY_LENGTH);
    
    /* Execute the key writing */
    if (writeUserKey(&s_key, u8_slotID)== ERR_LOG_SUCCESS)
    {
      GPIO_setP01State(GPIO_STATE_HIGH);
      printf("\nKey Write Successful");
    }
    else
    {
      GPIO_setP02State(GPIO_STATE_HIGH);
      printf("\nKey Write Failed");
    }
  }
  
  for (;;)
  {
    /* Main watchdog service */
    (void) PMU_serviceFailSafeWatchdog();
  }
}


/*
 * Function to convert the char to equivalent hex/integer value
 */
uint8 charToHex(uint8 charX){
  uint8 convHex;
  /* From 0x0 to 0x9 */
  if(charX >= 48  && charX < 64)
  {
    convHex = charX - 48;
  }
  /* From 0xa to 0xf */
  else if (charX >= 65  && charX <= 70)
  {
    convHex = charX - 55;
  }
  /* From 0xA to 0xF */
  else if (charX >= 97  && charX <= 102)
  {
    convHex = charX - 87;
  }
  /* Carriage return*/
  else if (charX == 0x0D )
  {
    convHex = 0;
  }
  /* default calue for all other cases */
  else
  {
    convHex = 0;
  }
  return convHex;
}
