/*
 ***********************************************************************************************************************
 *
 * Copyright (c) 2021-2023, Infineon Technologies AG
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 **********************************************************************************************************************/

/*******************************************************************************
**                             Author(s) Identity                             **
********************************************************************************
** Initials     Name                                                          **
** ---------------------------------------------------------------------------**
** BG           Blandine Guillot                                              **
** JO           Julia Ott                                                     **
** EE           Erich Englbrecht                                              **
*******************************************************************************/

/*******************************************************************************
**                          Revision Control History                          **
********************************************************************************
** V1.0.0: 2021-04-28, EE:   EP-700: Initial version                          **
** V1.0.1: 2021-07-27, BG:   EP-814: Added return code for TLE_init()         **
**                           Removed clearing of status registers, should be  **
**                           done in the init functions if they are passed    **
** V1.0.2: 2021-10-06, JO:   EP-949: Corrected memory layout (IROM1 size,     **
**                           IRAM1 start and size, IRAM2 size)                **
** V1.0.3: 2021-10-15, JO:   EP-962: Updated NAC time                         **
** V1.0.4: 2021-11-11, JO:   EP-937: Updated copyright and branding           **
** V1.0.5: 2021-12-20, JO:   EP-977: Added '--diag_suppress 1609' to Asm Misc **
**                           Control to remove compiler warning               **
** V1.1.0: 2023-04-18, JO:   EP-1235: Changed to C startup file               **
** V1.2.0: 2024-06-14, JO:   EP-1480: Disabled  warning 'unsafe buffer usage' **
**                           (ARM compiler for Embedded version >6.21)        **
*******************************************************************************/

/*******************************************************************************
**                                  Abstract                                  **
********************************************************************************
** SSC: Sending magic value via SSC0 and loopback to SSC1                     **
********************************************************************************
** SSC0 is configured as master, with clock output on P0.3 and serial output  **
** on P0.1. SSC1 is configured as slave, with clock input on P1.0 and serial  **
** input on P1.1. The clock is configured with 1MBaud.                        **
** Connect the clock pins and the input/output together to see the            **
** transmission/reception of the magic value. If the received value does not  **
** match the MAGIC_TRANSFER value, the GPIO pin P0.2 is set to high.          **                                                 **
** SSC0 (Master)                                                 SSC1 (Slave) **
** CLK  (P0.3) ------------------------------------------------> CLK  (P1.0)  **
** MOSI (P0.1) ------------------------------------------------> MOSI (P1.1)  **
*******************************************************************************/

/*******************************************************************************
**                                  Includes                                  **
*******************************************************************************/
#include "tle_device.h"
#include "types.h"
#include "gpio.h"

/*******************************************************************************
**                                 Constants                                  **
*******************************************************************************/
#define MAGIC_TRANSFER (0xC001CAFEBADC0DEDU)


sint32 main(void)
{
  sint8 s8_returnCode;
  
  /* Value to receive from SSC module */
  uint64 u64_receiveValue = 0u;

  /* Clear watchdog fail status */
  PMU_clrFailSafeWatchdogFailSts();

  /* Initialization of hardware modules based on Config Wizard configuration */
  s8_returnCode = TLE_init();
  
  if (s8_returnCode != ERR_LOG_SUCCESS)
  {
    /* Place your code here to handle an initialization error */
  }

  for (;;)
  {
    /* Main watchdog service */
    (void) PMU_serviceFailSafeWatchdog();

    /* Send the magic value with SSC0 */
    SSC0_setTXvalue((uint64) MAGIC_TRANSFER);
    
    /* Store the RX buffer value of SSC1 */
    u64_receiveValue = (uint64) (sint64) SSC1_getRXvalue();
    
    /* Compare received value with expected MAGIC_TRANSFER define */
    if (u64_receiveValue == MAGIC_TRANSFER) {
      /* In case of a match, the GPIO pin P0.2 is set to low */
      GPIO_setP02State(GPIO_STATE_LOW);
    } else {
      /* If there is no match, the GPIO pin P0.2 is set to high */
      GPIO_setP02State(GPIO_STATE_HIGH);
    }
  }
}
