/*
 ***********************************************************************************************************************
 *
 * Copyright (c) 2020-2023, Infineon Technologies AG
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 **********************************************************************************************************************/

/*******************************************************************************
**                             Author(s) Identity                             **
********************************************************************************
** Initials     Name                                                          **
** ---------------------------------------------------------------------------**
** BG           Blandine Guillot                                              **
** JO           Julia Ott                                                     **
** EE           Erich Englbrecht                                              *
*******************************************************************************/

/*******************************************************************************
**                          Revision Control History                          **
********************************************************************************
** V1.0.0: 2020-06-30, EE:   EP-856: Initial version                          **
** V1.0.1: 2021-10-06, JO:   EP-949: Corrected memory layout (IROM1 size,     **
**                           IRAM1 start and size, IRAM2 size)                **
** V1.0.2: 2021-10-15, JO:   EP-962: Updated NAC time                         **
** V1.0.3: 2021-10-22, BG:   EP-814: Added return code for TLE_init()         **
**                           Removed clearing of status registers, should be  **
**                           done in the init functions if they are passed    **
** V1.0.4: 2021-11-11, JO:   EP-937: Updated copyright and branding           **
** V1.0.5: 2021-12-20, JO:   EP-977: Added '--diag_suppress 1609' to Asm Misc **
**                           Control to remove compiler warning               **
** V1.1.0: 2023-04-18, JO:   EP-1235: Changed to C startup file               **
** V1.2.0: 2024-06-14, JO:   EP-1480: Disabled  warning 'unsafe buffer usage' **
**                           (ARM compiler for Embedded version >6.21)        **
*******************************************************************************/

/*******************************************************************************
**                                  Abstract                                  **
********************************************************************************
** UART: Send and receive data via J-Link pins using the UART                 **
********************************************************************************
** By starting this example a command line menu is printed on the console     **
** using the UART virtual com port of J-Link. The implemented command line    **
** handler offers three simple command to show the sending and receiving of   **
** data to and from the eval device.                                          **
*******************************************************************************/

/*******************************************************************************
**                                  Includes                                  **
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "tle_device.h"
#include "types.h"

/*******************************************************************************
**                                 Constants                                  **
*******************************************************************************/
#define BUFFER_SIZE (10U)

/*******************************************************************************
**                                 Variables                                  **
*******************************************************************************/
static volatile bool b_cmdTrigger = true;
static volatile uint8 u8_readCnt = 0;
static uint8 u8_buffer[BUFFER_SIZE];

/*******************************************************************************
**                           Function Declarations                            **
*******************************************************************************/
void handleCommand(void);


sint32 main(void)
{
  sint8 s8_returnCode;

  /* Clear watchdog fail status */
  PMU_clrFailSafeWatchdogFailSts();

  /* Initialization of hardware modules based on Config Wizard configuration */
  s8_returnCode = TLE_init();
  
  if (s8_returnCode != ERR_LOG_SUCCESS)
  {
    /* Place your code here to handle an initialization error */
  }

  for (;;)
  {
    /* Main watchdog service */
    (void) PMU_serviceFailSafeWatchdog();

    /* Check for activated command handler trigger */
    if (b_cmdTrigger == true)
    {
      /* Handle new command */
      handleCommand();
    }
  }
}


/* Command handler */
void handleCommand()
{
  /* Declare and initiate chip ID value */
  uint32 u32_chipId = 0;
  
  /* Check for initial call - only the menu must be printed */
  if (u8_readCnt != 0)
  {
    /* Check for 'cid' command */
    if (strcmp((void *) u8_buffer, "cid") == 0)
    {
      /* Read unique chip ID */
      user_cid_get(&u32_chipId);
      /* Send out chip ID via stdout */
      printf("\n --> CHIP ID 0x%x\n", u32_chipId);
    }
    /* Otherwise check for 'ack' command */
    else if (strcmp((void *) u8_buffer, "ack") == 0)
    {
      /* Send out acknowledgement via stdout */
      printf("\n --> SEND ACKnowledgement\n");
    }
    /* Otherwise check for 'nak' command */
    else if (strcmp((void *) u8_buffer, "nak") == 0)
    {
    }
    /* Otherwise unknown command */
    else
    {
      /* Send error message via stdout */
      printf("\n --> ERROR: command unknown\n");
    }
  }

  /* Print a fancy simple command menu */
  printf("\n*** Command menu  ***\n");
  printf(" <cid> - sends chip id\n");
  printf(" <ack> - sends ack\n");
  printf(" <nak> - keeps silent\n");
  printf("Enter a command:> ");

  /* Reset u8_readCnt for new command */
  u8_readCnt = 0;
  
  /* Deactivate command handler until new one is received */
  b_cmdTrigger = false;
}


/* UART receive ISR */
void uart_receive()
{
  /* Check for buffer overflow */
  if (u8_readCnt < BUFFER_SIZE)
  {
    /* Receive byte from P1.2 */
    u8_buffer[u8_readCnt] = (uint8) stdin_getchar();
    
    /* Echo byte to stdout to show character on console */
    printf("%c", u8_buffer[u8_readCnt]);
    
    /* Check if received character was a newline */
    if (u8_buffer[u8_readCnt] == 0x0d)
    {
      /* Add a final 0x00 to the string buffer */
      u8_buffer[u8_readCnt] = 0x00;
      
      /* Trigger command handler */
      b_cmdTrigger = true;
    }
    /* Increment receive counter for new character */
    u8_readCnt++;
  }
  else
  {
    /* Receive buffer is full -> handle command without newline */
    b_cmdTrigger = true;
  }
}
