/*
 ***********************************************************************************************************************
 *
 * Copyright (c) Infineon Technologies AG
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 **********************************************************************************************************************/
/**
 * \file     startup_tle989x.c
 *
 * \brief    startup file (c) for TLE988x/TLE989x
 *
 * \version  V0.1.4
 * \date     06. May 2024
 *
 * \note
 */

/*******************************************************************************
**                             Author(s) Identity                             **
********************************************************************************
** Initials     Name                                                          **
** ---------------------------------------------------------------------------**
** JO           Julia Ott                                                     **
** PS           Patrik Schwarz                                                **
** VO           Vanessa Ongaro                                                **
*******************************************************************************/

/*******************************************************************************
**                          Revision Control History                          **
********************************************************************************
** V0.1.0: 2022-08-18, JO:   EP-1235: Initial version of startup file in C    **
** V0.1.1: 2023-05-15, PS:   EP-1441: Update c startup file and linker file   **
**                                    for IAR to use sections                 **
** V0.1.2: 2023-08-10, VO:   EP-1464: Put debug_entry also in deprecated VTOR **
**                                    Fixed location of ER_NVM0_DEBUG_CONNECT **
**                                    in sct files                            **
** V0.1.3: 2023-11-29, VO:   EP-1465: Added chache enable to reset handler    **
** V0.1.4: 2024-05-06, VO:   EP-1510: Removed 'Todo' comments                 **
*******************************************************************************/

/* Work instructions:
   When using this startup file, the project needs a linker script.
   The linker script is has already been copied to the
   RTE/Device/--devicename-- folder of this project and
   needs to be added to the project manually.
   Keil:
   'Options for target'
   --> 'Linker' tab
   --> uncheck 'Use Memory Layout from Rarget Dialog'
   --> Scatter file: '...'
   --> add the linker file mentioned above (tle9881_91.sct or tle9883_93.sct)
*/

#include "tle989x.h"
#include "bootrom.h"
#include "tle_variants.h"
#include "bsl_defines.h"
#include "scu_defines.h"

/*---------------------------------------------------------------------------
  External References
 *---------------------------------------------------------------------------*/
extern uint32_t __INITIAL_SP;
extern uint32_t __STACK_LIMIT;
#if defined (__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE == 3U)
  extern uint32_t __STACK_SEAL;
#endif

/**
  \brief Exception / Interrupt Handler Function Prototype
*/
typedef void(*VECTOR_TABLE_Type)(void);

extern __NO_RETURN void __PROGRAM_START(void);

#if defined(__ARMCC_VERSION)
  /* Arm Compiler V5 and V6 */
  __attribute__((noreturn)) static void debug_entry(void) __attribute__((section("sec_NVM0_DEBUG_CONNECT"), used));
#elif defined(__IAR_SYSTEMS_ICC__)
  /* IAR compiler */
  __attribute__((noreturn)) static void debug_entry(void) @ "sec_NVM0_DEBUG_CONNECT";
#elif defined(UNIT_TESTING_OCN)
  __attribute__((noreturn)) static void debug_entry(void);
#endif

/*---------------------------------------------------------------------------
  Internal References
 *---------------------------------------------------------------------------*/
__NO_RETURN void Reset_Handler  (void);
__NO_RETURN void Default_Handler(void);
__NO_RETURN static void debug_entry(void);

/*---------------------------------------------------------------------------
  Exception / Interrupt Handler
 *---------------------------------------------------------------------------*/
/* Exceptions */
void NMI_Handler            (void) __attribute__ ((weak, alias("Default_Handler")));
void HardFault_Handler      (void) __attribute__ ((weak));
void MemManage_Handler      (void) __attribute__ ((weak, alias("Default_Handler")));
void BusFault_Handler       (void) __attribute__ ((weak, alias("Default_Handler")));
void UsageFault_Handler     (void) __attribute__ ((weak, alias("Default_Handler")));
//void SecureFault_Handler    (void) __attribute__ ((weak, alias("Default_Handler")));
void SVC_Handler            (void) __attribute__ ((weak, alias("Default_Handler")));
void DebugMon_Handler       (void) __attribute__ ((weak, alias("Default_Handler")));
void PendSV_Handler         (void) __attribute__ ((weak, alias("Default_Handler")));
void SysTick_Handler        (void) __attribute__ ((weak, alias("Default_Handler")));

void NVIC_IRQ0_Handler(void)   __attribute__ ((weak, alias("Default_Handler"))); // 0: PMU, BDRV, CSC, ARVG
void NVIC_IRQ1_Handler(void)   __attribute__ ((weak, alias("Default_Handler"))); // 1: PMU, BDRV, CSC, ARVG
void NVIC_IRQ2_Handler(void)   __attribute__ ((weak, alias("Default_Handler"))); // 2: CCU7
void NVIC_IRQ3_Handler(void)   __attribute__ ((weak, alias("Default_Handler"))); // 3: CCU7
void NVIC_IRQ4_Handler(void)   __attribute__ ((weak, alias("Default_Handler"))); // 4: CCU7
void NVIC_IRQ5_Handler(void)   __attribute__ ((weak, alias("Default_Handler"))); // 5: CCU7
//0x000000c1 // 6: MEMCTRL
void NVIC_IRQ7_Handler(void)  __attribute__ ((weak, alias("Default_Handler"))); // 7: GPT12E
void NVIC_IRQ8_Handler(void)  __attribute__ ((weak, alias("Default_Handler"))); // 8: GPT12E
void NVIC_IRQ9_Handler (void) __attribute__ ((weak, alias("Default_Handler"))); // 9: Timer2
void NVIC_IRQ10_Handler(void) __attribute__ ((weak, alias("Default_Handler"))); // 10: ADC2
void NVIC_IRQ11_Handler(void) __attribute__ ((weak, alias("Default_Handler"))); // 11: ADC2
void NVIC_IRQ12_Handler(void) __attribute__ ((weak, alias("Default_Handler"))); // 12: MON
void NVIC_IRQ13_Handler(void) __attribute__ ((weak, alias("Default_Handler"))); // 13: MON
void NVIC_IRQ14_Handler(void) __attribute__ ((weak, alias("Default_Handler"))); // 14: ADC1
void NVIC_IRQ15_Handler(void) __attribute__ ((weak, alias("Default_Handler"))); // 15: ADC1
void NVIC_IRQ16_Handler(void) __attribute__ ((weak, alias("Default_Handler"))); // 16: ADC1
void NVIC_IRQ17_Handler(void) __attribute__ ((weak, alias("Default_Handler"))); // 17: ADC1
void NVIC_IRQ18_Handler(void) __attribute__ ((weak, alias("Default_Handler"))); // 18: BEMF, SDADC
void NVIC_IRQ19_Handler(void) __attribute__ ((weak, alias("Default_Handler"))); // 19: BEMF, SDADC
void NVIC_IRQ20_Handler(void) __attribute__ ((weak, alias("Default_Handler"))); // 20: EXTINT
void NVIC_IRQ21_Handler(void) __attribute__ ((weak, alias("Default_Handler"))); // 21: EXTINT
void NVIC_IRQ22_Handler(void) __attribute__ ((weak, alias("Default_Handler"))); // 22: UART0/1, LIN
void NVIC_IRQ23_Handler(void) __attribute__ ((weak, alias("Default_Handler"))); // 23: UART0/1, LIN
void NVIC_IRQ24_Handler(void) __attribute__ ((weak, alias("Default_Handler"))); // 24: SSC0/1
void NVIC_IRQ25_Handler(void) __attribute__ ((weak, alias("Default_Handler"))); // 25: SSC0/1
void NVIC_IRQ26_Handler(void) __attribute__ ((weak, alias("Default_Handler"))); // 26: MultiCAN
void NVIC_IRQ27_Handler(void) __attribute__ ((weak, alias("Default_Handler"))); // 27: MultiCAN
void NVIC_IRQ28_Handler(void) __attribute__ ((weak, alias("Default_Handler"))); // 28: MultiCAN
void NVIC_IRQ29_Handler(void) __attribute__ ((weak, alias("Default_Handler"))); // 29: DMA
void NVIC_IRQ30_Handler(void) __attribute__ ((weak, alias("Default_Handler"))); // 30: DMA
void NVIC_IRQ31_Handler(void) __attribute__ ((weak, alias("Default_Handler"))); // 31: Timer21

/*----------------------------------------------------------------------------
  Exception / Interrupt Vector table
 *----------------------------------------------------------------------------*/

#if defined ( __GNUC__ )
  #pragma GCC diagnostic push
  #pragma GCC diagnostic ignored "-Wpedantic"
#endif

#if defined(__ARMCC_VERSION)
  /* Arm Compiler V5 and V6 */
  static const uint32 u32_flash0_page0[11] __attribute__((section("sec_NVM0_BOOTCFG"), used)) =
#elif defined(__IAR_SYSTEMS_ICC__)
  /* IAR compiler */
  __root const uint32 u32_flash0_page0[11] @ "sec_NVM0_BOOTCFG" =
#elif defined(UNIT_TESTING_OCN)
  const uint32 u32_flash0_page0[11] =
#endif
{
  (uint32)BSL_NAD_NAC,   /* 0x00: NAC,NAD */
  0xFFFFFFFF,            /* 0x04: reserved */
  0xFFFFFFFF,            /* 0x08: Secure Boot Hash */
  0xFFFFFFFF,            /* 0x0C: Secure Boot Hash */
  0xFFFFFFFF,            /* 0x10: Secure Boot Hash */
  0xFFFFFFFF,            /* 0x14: Secure Boot Hash */
  0xFFFFFFFF,            /* 0x18: reserved */
  0xFFFFFFFF,            /* 0x1C: reserved */
  0xFFFFFFFF,            /* 0x20: reserved */
  0xFFFFFFFF,            /* 0x24: User BSL Code Size */
  0xFFFFFFFF,            /* 0x28: Secure Key ID */
};

#if defined(__ARMCC_VERSION)
  /* Arm Compiler V5 and V6 */
  static const VECTOR_TABLE_Type vt_vectab_deprecated[2] __attribute__((section("sec_NVM0_VECTAB_DEPRECATED"), used)) =
#elif defined(__IAR_SYSTEMS_ICC__)
  /* IAR compiler: __root needed so that the variable is not optimized away */
  __root const VECTOR_TABLE_Type vt_vectab_deprecated[2] @ "sec_NVM0_VECTAB_DEPRECATED" =
#elif defined(UNIT_TESTING_OCN)
  static const VECTOR_TABLE_Type vt_vectab_deprecated[2] =
#endif
{
  (VECTOR_TABLE_Type)(&__INITIAL_SP),  /*     Initial Stack Pointer */
  debug_entry,                         /*     Reset Handler */
};

__attribute__((noreturn)) static void debug_entry(void)
{
#define SCU_BOOT_CFG_LATCH_ADDR 0x480040BC
#define SCU_BOOT_CFG_LATCH_TMS_MASK 0x00000001
#define SCU_BOOT_CFG_LATCH_P00_MASK 0x00000002
#define CPU_DHCSR_ADDR 0xE000EDF0
#define CPU_DHCSR_C_DEBUGEN_MASK 0x00000001
  uint32 u32_boot_cfg_latch;
  uint32 u32_dhcsr_c_debugen;
  u32_boot_cfg_latch = *(uint32 *)SCU_BOOT_CFG_LATCH_ADDR;

  if ((u32_boot_cfg_latch & (SCU_BOOT_CFG_LATCH_TMS_MASK | SCU_BOOT_CFG_LATCH_P00_MASK)) == (SCU_BOOT_CFG_LATCH_TMS_MASK | SCU_BOOT_CFG_LATCH_P00_MASK))
  {
    do
    {
      u32_dhcsr_c_debugen = (*(uint32 *)CPU_DHCSR_ADDR) & CPU_DHCSR_C_DEBUGEN_MASK;
    }
    while (u32_dhcsr_c_debugen == 0);
  }

  Reset_Handler();
}

extern const VECTOR_TABLE_Type __VECTOR_TABLE[48];

#if defined(__ARMCC_VERSION)
/* Arm Compiler V5 and V6 */
const VECTOR_TABLE_Type __VECTOR_TABLE[48] __VECTOR_TABLE_ATTRIBUTE =
{
#elif defined(__IAR_SYSTEMS_ICC__)
/* IAR compiler */
const VECTOR_TABLE_Type __VECTOR_TABLE[48] @ "sec_NVM0_VECTAB" = {
#elif defined(UNIT_TESTING_OCN)
const VECTOR_TABLE_Type __VECTOR_TABLE[48] = {
#endif
  (VECTOR_TABLE_Type)(&__INITIAL_SP),  /*     Initial Stack Pointer */
  debug_entry,                         /*     Reset Handler */
  NMI_Handler,                         /* -14 NMI Handler */
  HardFault_Handler,                   /* -13 Hard Fault Handler */
  MemManage_Handler,                   /* -12 MPU Fault Handler */
  BusFault_Handler,                    /* -11 Bus Fault Handler */
  UsageFault_Handler,                  /* -10 Usage Fault Handler */
  0,                                   /*  -9 Secure Fault Handler */
  0,                                   /*     Reserved */
  0,                                   /*     Reserved */
  0,                                   /*     Reserved */
  SVC_Handler,                         /*  -5 SVCall Handler */
  DebugMon_Handler,                    /*  -4 Debug Monitor Handler */
  0,                                   /*     Reserved */
  PendSV_Handler,                      /*  -2 PendSV Handler */
  SysTick_Handler,                     /*  -1 SysTick Handler */

  /* Interrupts */
  NVIC_IRQ0_Handler,             // 0: PMU, BDRV, CSC, ARVG
  NVIC_IRQ1_Handler,             // 1: PMU, BDRV, CSC, ARVG
  NVIC_IRQ2_Handler,             // 2: CCU7
  NVIC_IRQ3_Handler,             // 3: CCU7
  NVIC_IRQ4_Handler,             // 4: CCU7
  NVIC_IRQ5_Handler,             // 5: CCU7
  (VECTOR_TABLE_Type)0x000000c1, // 6: MEMCTRL
  NVIC_IRQ7_Handler,             // 7: GPT12E
  NVIC_IRQ8_Handler,             // 8: GPT12E
  NVIC_IRQ9_Handler,             // 9: Timer2
  NVIC_IRQ10_Handler,            // 10: ADC2
  NVIC_IRQ11_Handler,            // 11: ADC2
  NVIC_IRQ12_Handler,            // 12: MON
  NVIC_IRQ13_Handler,            // 13: MON
  NVIC_IRQ14_Handler,            // 14: ADC1
  NVIC_IRQ15_Handler,            // 15: ADC1
  NVIC_IRQ16_Handler,            // 16: ADC1
  NVIC_IRQ17_Handler,            // 17: ADC1
  NVIC_IRQ18_Handler,            // 18: BEMF, SDADC
  NVIC_IRQ19_Handler,            // 19: BEMF, SDADC
  NVIC_IRQ20_Handler,            // 20: EXTINT
  NVIC_IRQ21_Handler,            // 21: EXTINT
  NVIC_IRQ22_Handler,            // 22: UART0/1, LIN
  NVIC_IRQ23_Handler,            // 23: UART0/1, LIN
  NVIC_IRQ24_Handler,            // 24: SSC0/1
  NVIC_IRQ25_Handler,            // 25: SSC0/1
  NVIC_IRQ26_Handler,            // 26: MultiCAN
  NVIC_IRQ27_Handler,            // 27: MultiCAN
  NVIC_IRQ28_Handler,            // 28: MultiCAN
  NVIC_IRQ29_Handler,            // 29: DMA
  NVIC_IRQ30_Handler,            // 30: DMA
  NVIC_IRQ31_Handler             // 31: Timer21
};

#if defined ( __GNUC__ )
#pragma GCC diagnostic pop
#endif

/*---------------------------------------------------------------------------
  Reset Handler called on controller reset
 *---------------------------------------------------------------------------*/
__NO_RETURN void Reset_Handler(void)
{
  __set_PSP((uint32_t)(&__INITIAL_SP));
#if defined (__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE == 3U)
  __TZ_set_STACKSEAL_S((uint32_t *)(&__STACK_SEAL));
#endif

  /* Enable Flash1 caching according to CW setting */
  if(FLASH1_CACHE_EN == 1)
  {
    user_cache_operation(CACHE_OP_EN, UC_FLASH1_START);
  }
  
  SystemInit();                    /* CMSIS System Initialization */
  __PROGRAM_START();               /* Enter PreMain (C library entry point) */
}

#if defined(__ARMCC_VERSION) && (__ARMCC_VERSION >= 6010050)
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wmissing-noreturn"
#endif

/*---------------------------------------------------------------------------
  Hard Fault Handler
 *---------------------------------------------------------------------------*/
void HardFault_Handler(void)
{
  while (1);
}

/*---------------------------------------------------------------------------
  Default Handler for Exceptions / Interrupts
 *---------------------------------------------------------------------------*/
void Default_Handler(void)
{
  while (1);
}

#if defined(__ARMCC_VERSION) && (__ARMCC_VERSION >= 6010050)
#pragma clang diagnostic pop
#endif
