/**************************************************************************************************
*                _  _               ___             _                                             *
*               (_)| |__   _ __    / _ \ _ __ ___  | |__    /\  /\                                *
*               | || '_ \ | '__|  / /_\/| '_ ` _ \ | '_ \  / /_/ /                                *
*               | || | | || |    / /_\\ | | | | | || |_) |/ __  /                                 *
*               |_||_| |_||_|    \____/ |_| |_| |_||_.__/ \/ /_/                                  *
*                                                                                                 *
*    ihr GmbH                                                                                     *
*    Airport Boulevard B210                                                                       *
*    77836 Rheinmuenster - Germany                                                                *
*    http://www.ihr.de                                                                            *
*    Phone +49(0) 7229-18475-0                                                                    *
*    Fax   +49(0) 7229-18475-11                                                                   *
***************************************************************************************************
*                                                                                                 *
*       Revision:          $Rev:: 274         $                                                   *
*       Last Modtime:      $Date:: 2022-05-20#$                                                   *
*       Last Modification: $Author:: PhilippKoch                                                $ *
*                                                                                                 *
**************************************************************************************************/
/**
@file         Can_Api.h
@brief        Application interface declaration for Can driver
*/
#ifndef CAN_API_H 
#define CAN_API_H

/* ========== [includes] ======================================================================= */

#include "can_types.h"                      /* Com basic types needed for config */
#include "can_cfg.h"


/* ========== [compiler switches] ============================================================== */

/* empty */

/* ========== [defines] ======================================================================== */

/* empty */
   
/* ========== [macros] ========================================================================= */

/* empty */

/* ========== [unions, structs and enums] ====================================================== */

/* empty */

/* ========== [external function declarations] ================================================= */

/* ---------- initialization, deinitialization, state change ----- */
/**
   @function   Can_Init
   @brief      Initialization of Can driver. 
   @par        This function has to be called once during startup and once after every wake up. 
               All module variables and states are initialized.
               All hardware registers and the message RAM are initialized from the CAN configuration.
   @param      void
   @retval     void
*/
extern void Can_Init(void);

/**
   @function   Can_DeInit
   @brief      This function shall be called only during shutdown routine
   @par        This function stops all CAN communication and resets the CAN controller
               No callouts will be called after DeInit.
   @param      void
   @retval     void
*/
extern void Can_DeInit(void);

/**
   @function   Can_ControllerReset
   @brief      trigger a specific CAN controller channel to re-init the hardware after bus-off
   @par        This function restarts communication on a CAN channel to recover from a bus-off event.
   @param      Channel: the controller channel index which shall be reset
   @retval     result of operation: 
               E_OK:            operation successful
			   E_NOT_OK:        parameter out of range 
*/
extern uint8 Can_ControllerReset( uint8 Channel );

/* ---------- cyclic runnables ----- */
/**
   @function   Can_MainFunctionWrite
   @brief      Runnable of Can module
   @par        Main function to be called cyclic in polling mode, not necessarily in constant time.
               Processes all pending transmit actions on the hardware
			   - check if a transmission was successful and callback Com_TxConfirmation(.., E_OK)
			   - check if transmissions finally failed and callback Com_TxConfirmation(.., E_NOT_OK)
   @param      void 
   @retval     void
*/
extern void Can_MainFunctionWrite(void);

/**
   @function   Can_MainFunctionRead
   @brief      Runnable of Can module: cyclic call required when receive path operates in polling mode.
   @par        Main function to be called cyclic in polling mode, not necessarily in constant time.
               Processes all recently received frames on the hardware
			   - check if fifos or CAN objects have received a frame, get the CAN ID.
			   - lookup the Rx handle in the associated config table by ID search.
			   - forward received frame indications to Com by calling Com_RxIndication for each received frame.
			   - discard unknown frames from which CAN ID is not found in the config table.
   @param      void
   @retval     void
*/
extern void Can_MainFunctionRead(void);


/* ---------- sending/receiving ----- */
/**
   @function   Can_Transmit
   @brief      Copy a data string and trigger transmission
   @par        The function copies data to the transmit message object with given length and sets the Tx request.
               The given length must not exceed the message hardware object size (8 bytes for standard CAN, 64 bytes for CAN-FD)
			   The CAN driver is using the CAN configuration to resolve which CAN channel and transmit method shall be used.
			   This can either be a FullCAN object or one of the Tx Fifos.
			   If a Fifo is full, the transmit request is discarded and CAN_BUSY is returned. The caller may then repeat the Tx request.
			   If a FullCAN object is still pending to be sent, it is updated (actually overwritten) and E_OK is returned.
   @param      TxFrameHdl: a handle index from the Tx frames list
               DataPtr: a pointer to the data source array
               Length: number of bytes to be copied, limited to the size of the frame buffer.
   @retval     result of operation: 
               E_OK:            operation successful
			   E_NOT_OK:        parameter out of range 
			   E_NOT_AVAILABLE: driver not initialized or in wrong state
			   E_BUSY:          BasicCAN Fifo is full, repeat request later
			   
*/
extern uint8 Can_Transmit( tComFrameHandle TxFrameHdl, const uint8* DataPtr, uint8 Length );



#if (CAN_USE_INTERRUPTS_FULLCAN_RX == STD_ON)
uint8 Can_Interrupts_Disable(void);
void  Can_Interrupts_Enable(uint8 previousState);
#endif



/* ========== [external variable declarations] ================================================= */

/* empty */

/* ========== [end of file] ==================================================================== */
#endif /* end #ifndef CAN_API_H */
