/**************************************************************************************************
*                _  _               ___             _                                             *
*               (_)| |__   _ __    / _ \ _ __ ___  | |__    /\  /\                                *
*               | || '_ \ | '__|  / /_\/| '_ ` _ \ | '_ \  / /_/ /                                *
*               | || | | || |    / /_\\ | | | | | || |_) |/ __  /                                 *
*               |_||_| |_||_|    \____/ |_| |_| |_||_.__/ \/ /_/                                  *
*                                                                                                 *
*    ihr GmbH                                                                                     *
*    Airport Boulevard B210                                                                       *
*    77836 Rheinmuenster - Germany                                                                *
*    http://www.ihr.de                                                                            *
*    Phone +49(0) 7229-18475-0                                                                    *
*    Fax   +49(0) 7229-18475-11                                                                   *
***************************************************************************************************
*                                                                                                 *
*       Revision:          $Rev:: 274         $                                                   *
*       Last Modtime:      $Date:: 2022-05-20#$                                                   *
*       Last Modification: $Author:: PhilippKoch                                                $ *
*                                                                                                 *
**************************************************************************************************/
/**
@file         Com_Api.h
@brief        Application interface declaration for Com layer
*/
#ifndef COM_API_H 
#define COM_API_H

/* ========== [includes] ======================================================================= */

#include "com_types.h"                      /* Com basic types needed for config */


/* ========== [compiler switches] ============================================================== */

/* empty */

/* ========== [defines] ======================================================================== */

/* empty */
   
/* ========== [macros] ========================================================================= */

/* empty */

/* ========== [unions, structs and enums] ====================================================== */

/* empty */

/* ========== [external function declarations] ================================================= */
/**
   @function   Com_InitRx
   @brief      This function initializes the RX path of the Com layer.
   @param      void
   @retval     void
*/
extern void Com_InitRx(void);

/**
   @function   Com_InitTx
   @brief      This function initializes the TX path of the Com layer.
   @param      void
   @retval     void
*/
extern void Com_InitTx(void);


/* ---------- initialization, deinitialization, state change ----- */
/**
   @function   Com_Init
   @brief      Initialization of Com module. 
   @par        This function has to be called once during startup and once after every wake up. 
               All module variables and states are initialized.
               All data buffers and shadow buffers are initialized with start values.
			   Transmission is disabled. Reception deadline monitoring is disabled.
			   Receive path and receive indication is enabled, 
			   i.e. to enable network management and wakeup communication control
   @param      void
   @retval     void
*/
extern void Com_Init(void);

/**
   @function   Com_DeInit
   @brief      This function shall be called only during shutdown routine
   @par        This function stops all Com channels communication and frees resources
               No callouts will be called after DeInit.
   @param      void
   @retval     void
*/
extern void Com_DeInit(void);

/**
   @function   Com_EnableTransmission
   @brief      enables/disables Tx path to hardware driver 
   @par        This function has to be called once during startup and once after every wake up. 
               Transmit path can be disabled to keep the network silent, i.e. by network management, prepare to sleep.
   @param      bEnable (STD_ON, STD_OFF)
   @retval     void
*/
extern void Com_EnableTransmission(uint8 bEnable);

/**
   @function   Com_EnableReceptionDM
   @brief      enables/disables Rx frame deadline monitoring 
   @par        This function has to be called once during startup and once after every wake up. 
               Receive deadline monitoring can be disabled while the network is expected to be silent, i.e. by network management, prepare to sleep. No timeout faults are raised then.
			   Note: When deadline monitoring is enabled, all timeout counters are set to their initial values.
			   If DM was already enabled before, all counters are still reset to initial values.
   @param      bEnable (STD_ON, STD_OFF)
   @retval     void
*/
extern void Com_EnableReceptionDM(uint8 bEnable);



/* ---------- cyclic runnables ----- */
/**
   @function   Com_MainFunctionTx
   @brief      Runnable of Com module (<call cycle>)
   @par        Main function to be called in constant timed call cycle.
               Processes all pending actions for message transmission from Com layer to the hardware driver.
			   - verify channel status if enabled / disabled
			   - progress on cycle timing counters and generate transmit requests when counters elapsed
			   - check pending transmit requests and forward message to HW driver
   @param      void
   @retval     void
*/
extern void Com_MainFunctionTx(void);

/**
   @function   Com_MainFunctionRx
   @brief      Runnable of Com module (<call cycle>)
   @par        Main function to be called in constant timed call cycle.
               Processes all received messages from hardware driver to Com layer.
			   - verify channel status if enabled / disabled
			   - progress on timeout supervising counters and generate timeout errors
			   - check for received messages, store in buffers and indicate to the application
   @param      void
   @retval     void
*/
extern void Com_MainFunctionRx(const uint8 taskMs);

/* ---------- signal based sending/receiving ----- */
/**
   @function   Com_SendSignal
   @brief      Write a signal value to the transmit buffer
   @par        The application provides a signal value which is read from the DataPtr location.
               The type is determined by the signal configuration list.
               If no Shadow buffer is defined, the sigal value is packed into the Frame Buffer.
			   If a Shadow Buffer is defined, the signal value is packed into the Shadow Buffer.
			   A message transmission is NOT implicitly triggered by sending a signal.
   @param      TxSignalHdl: a handle index from the Tx signals list
               SignalDataPtr: a void* to the source value. The type can be u8, u16, u32, 
			   depending on the configuration for this signal
   @retval     result of operation: 
               E_OK, E_NOT_OK, E_NOT_AVAILABLE, E_BUSY
*/
extern uint8 Com_SendSignal( tComSignalHandle TxSignalHdl, const void * const SignalDataPtr );


/**
   @function   Com_SendSignalGroup
   @brief      Finalize sending signals by updating the frame buffer
   @par        This function copies the content of the shadow buffer to the frame buffer.
               if a shadow buffer is configured for this frame,
			   this function must be called after all signals were updated by calls to Com_SendSignal.
			   The purpose of sending signals to a shadow buffer is to keep the whole signal group atomic while updating it.
			   Note: Signal groups are currently not supported. 
			         The owner frame handle is defined to replace the signal group ID.
   @param      TxFrameHdl: a handle index from the Tx frames list
   @retval     result of operation: 
               E_OK, E_NOT_OK, E_NOT_AVAILABLE, E_BUSY
*/
extern uint8 Com_SendSignalGroup( tComFrameHandle TxFrameHdl );


/**
   @function   Com_ReceiveSignal
   @brief      Read a signal value from the message receive buffer
   @par        The application provides a signal storage variable which is updated from the message buffer.
               The type is determined by the signal configuration list. 
               If no Shadow buffer is defined, the sigal value is unpacked from the Frame Buffer.
			   If a Shadow Buffer is defined, the signal value is unpacked from the Shadow Buffer.
			   If the signal value type is signed, and the signal size is not 8 or 16 or 32, then sign extension is applied to the signal value.
   @param      RxSignalHdl: a handle index from the Rx signals list
               SignalDataPtr: a void* to the destination variable. The type can be u8, i8, u16, i16, u32, i32. 
   @retval     result of operation: 
               E_OK, E_NOT_OK, E_NOT_AVAILABLE, E_BUSY
*/
extern uint8 Com_ReceiveSignal( tComSignalHandle RxSignalHdl, void * const SignalDataPtr );


/**
   @function   Com_ReceiveSignalGroup
   @brief      Prepare receiving signals from the shadow buffer
   @par        This function is called to copy the content of the frame buffer to the shadow buffer.
   		       If a shadow buffer is configured for this frame, 
			   this function must be called before the signals can be received by calls to Com_ReceiveSignal.
			   The purpose of receiving signals from a shadow buffer is to keep the whole signal group atomic while reading signal values from it.
			   Note: Signal groups are currently not supported. 
			         The owner frame handle is defined to replace the signal group ID.
   @param      RxFrameHdl: a handle index from the Rx frames list
   @retval     result of operation: 
               E_OK, E_NOT_OK, E_NOT_AVAILABLE, E_BUSY
*/
extern uint8 Com_ReceiveSignalGroup( tComFrameHandle TxFrameHdl );


/* ---------- array based sending/receiving ----- */
/**
   @function   Com_SendData
   @brief      Copy a data string for transmission
   @par        The function copies data from the application source to the transmit frame buffer.
               The given length should match the size of the associated frame buffer, but can be smaller, but never greater.
			   A message transmission is NOT implicitly triggered by sending data.
   @param      TxFrameHdl: a handle index from the Tx frames list
               DataPtr: a pointer to the data source array
               Length: number of bytes to be copied, limited to the size of the frame buffer.
   @retval     result of operation: 
               E_OK, E_NOT_OK, E_NOT_AVAILABLE, E_BUSY
*/
extern uint8 Com_SendData( tComFrameHandle TxFrameHdl, const uint8* DataPtr, uint8 Length );


/**
   @function   Com_ReceiveData
   @brief      Receive data and copy to the application data array
   @par        The function copies data from the receive frame buffer to the application destination.
               The given length should match the size of the associated frame buffer, but can be smaller, but never greater.
   @param      RxFrameHdl: a handle index from the Rx frames list
               DataPtr: a pointer to the data destination array
               Length: number of bytes to be copied, limited to the size of the frame buffer.
   @retval     result of operation: 
               E_OK, E_NOT_OK, E_NOT_AVAILABLE, E_BUSY
*/
extern uint8 Com_ReceiveData( tComFrameHandle RxFrameHdl, uint8* DataPtr, uint8 Length );



/**
   @function   Com_Transmit
   @brief      Trigger an explicit transmit request for a Tx frame
   @par        This function is called to initiate a transmission of the frame with disregard to its transmit cycle.
               If a cycle time is configured for the frame, the cycle counter is restarted after the frame has been sent.
   		       This function shall not be called if transmission is based on fixed cycle only.
   @param      TxFrameHdl: a handle index from the Tx frames list
   @retval     result of operation: 
               E_OK, E_NOT_OK, E_NOT_AVAILABLE, E_BUSY
*/
extern uint8 Com_Transmit( tComFrameHandle TxFrameHdl );


/* ---------- callout functions from Can layer ----- */
/**
   @function   Com_TxConfirmation
   @brief      Confirm a successful transmission or a failed transmission attempt by the Can driver
   @par        This function is a callout from Can driver to confirm a successful or failed transmission of the frame that had been Tx requested before.
   		       Com can forward the confirmation to the upper layer in case if a callout function is defined for this frame.
			   Tx confirmation can be used to control Tx timeout supervising or communication state machines.
			   In case of success, Com calls the upper layer TxConfirmation function
			   In case of failure, Com calls the upper layer TxError function
   @param      TxFrameHdl: a handle index from the Tx frames list
               Status: result of the transmit request; E_OK = success, E_NOT_OK = failed
   @retval     void
*/
extern void Com_TxConfirmation( tComFrameHandle TxFrameHdl, uint8 Status );

/**
   @function   Com_RxIndication
   @brief      Indicate a received frame by the Can driver
   @par        This function is a callout from Can driver to indicate a successful reception of the frame.
   		       Com shall check the length first, if the received data fits in the Com buffer.
			   If the data is too long and a receive error notification callout function is defined, 
			      receiving fails and a receive error is forwarded to the upper layer.
			   Else: received data is copied to Com buffer.
			   Receive deadline monitoring is then performed for the frame (restart of timeout counter).
			   If a receive indication callout function is defined for the frame, Com forwards the Rx indication to the upper layer.
   @param      RxFrameHdl: a handle index from the Tx frames list
               DataPtr:    pointer to received data in can hardware. 
			     Note: after returning from this function, the pointer becomes invalid as the message object is freed. 
				 So the RxIndication function must copy the data before returning.
               Length: 	   length of received frame according to frame DLC (expanded to number of bytes if CAN-FD)
   @retval     void
*/
extern void Com_RxIndication(const tComFrameHandle RxFrameHdl, const uint8* DataPtr, const uint8 Length);

/**
   @function   Com_ControllerBusOff
   @brief      Can driver notifying that a CAN controller has gone bus-off
   @par        This function is a callout from Can driver to notify a bus-off event of a controller.
               After bus-off, Com may call the error notification to the application, 
                 and may wait some time before calling Can_ControllerReset.
   @param      Channel: the channel index which has gone bus-off
   @retval     void
*/
extern void Com_ControllerBusOff(uint8 Channel);


/*
Example for a Com lifecycle

At startup:
   Com_Init();
   

When network management decides to enable communication (with no NM: at startup):
   Com_EnableTransmission(STD_ON);
   Com_EnableReceptionDM(STD_ON);


When a receive notification is called to the application:

OnReceiveIndicateMyFrameXYZ(uint16 RxHdl, uint8 length) {
   a) receiving as byte array
   Com_ReceiveData(ComRx_MyFrameXYZ, &MyLocalDataBuffer, sizeof(MyLocalDataBuffer);
   
   b) receiving signals without double buffering:
   Com_ReceiveSignal(ComRxSig_MyXYZsignal01, &MyXYZsignalVariable01);
   Com_ReceiveSignal(ComRxSig_MyXYZsignal02, &MyXYZsignalVariable02); ...
   
   c) receiving double buffered signals:
   Com_ReceiveSignalGroup(ComRx_MyFrameXYZ);   
   Com_ReceiveSignal(ComRxSig_MyXYZsignal03, &MyXYZsignalVariable03);
   Com_ReceiveSignal(ComRxSig_MyXYZsignal04, &MyXYZsignalVariable04); ...
}
   

When application prepares data to send:
   a) sending as byte array
   Com_SendData(ComTx_MyFrameABC, &MyLocalDataBuffer, sizeof(MyLocalDataBuffer);
   Com_Transmit(ComTx_MyFrameABC);   // explicit transmit request if needed
   
   b) sending signals without double buffering
   Com_SendSignal(ComTxSig_MyABCsignal01, &MyABCsignalVariable01);
   Com_SendSignal(ComTxSig_MyABCsignal02, &MyABCsignalVariable02); ...
   Com_Transmit(ComTx_MyFrameABC);   // explicit transmit request if needed
   
   c) sending double buffered signals atomic
   Com_SendSignal(ComTxSig_MyABCsignal01, &MyABCsignalVariable01);
   Com_SendSignal(ComTxSig_MyABCsignal02, &MyABCsignalVariable02); ...
   Com_SendSignalGroup(ComTx_MyFrameABC);
   Com_Transmit(ComTx_MyFrameABC);   // explicit transmit request if needed


When network management decides to wait for network silent:
   Com_EnableTransmission(STD_OFF);
   Com_EnableReceptionDM(STD_OFF);

When network management has detected sleeping network and prepares for power-off:
   Com_DeInit();
*/

/* ========== [external variable declarations] ================================================= */

/* empty */

/* ========== [end of file] ==================================================================== */
#endif /* end #ifndef COM_API_H */
